from ya_market_api.base.dataclass import BaseResponse
from ya_market_api.feedback.const import CommentStatus
from ya_market_api.feedback.dataclass.generic import FeedbackCommentAuthor

from typing import Optional, Collection, FrozenSet, Dict, Any, List, ClassVar, overload
from warnings import warn

from pydantic.main import BaseModel
from pydantic.fields import Field
from pydantic.config import ConfigDict


class Request(BaseModel):
	QUERY_PARAMS: ClassVar[FrozenSet[str]] = frozenset({"limit", "page_token"})
	S11N_ALIAS_COMMENT_IDS: ClassVar[str] = "commentIds"
	model_config = ConfigDict(arbitrary_types_allowed=True)

	# query params
	limit: Optional[int] = None
	page_token: Optional[str] = None

	# payload
	feedback_id: Optional[int] = Field(default=None, serialization_alias="feedbackId")
	comment_ids: Optional[Collection[int]] = Field(
		default=None,
		serialization_alias=S11N_ALIAS_COMMENT_IDS,
		min_length=1,
		max_length=50,
	)

	@overload
	def __init__(
		self,
		*,
		comment_ids: Collection[int],
		limit: Optional[int] = None,
		page_token: Optional[str] = None,
	) -> None: ...
	@overload
	def __init__(
		self,
		*,
		feedback_id: int,
		limit: Optional[int] = None,
		page_token: Optional[str] = None,
	) -> None: ...
	def __init__(
		self,
		*,
		comment_ids: Optional[Collection[int]] = None,
		feedback_id: Optional[int] = None,
		limit: Optional[int] = None,
		page_token: Optional[str] = None,
	) -> None:
		super().__init__(
			limit=limit,
			page_token=page_token,
			comment_ids=comment_ids,
			feedback_id=feedback_id,
		)

	def model_dump_request_params(self) -> Dict[str, Any]:
		return self.model_dump(include=self.QUERY_PARAMS, by_alias=True, exclude_none=True)

	def model_dump_request_payload(self) -> Dict[str, Any]:
		result = self.model_dump(exclude=self.QUERY_PARAMS, by_alias=True, exclude_none=True)

		if self.comment_ids is not None and len(result) != 1:
			warn("When using comment_ids, the other parameters will be ignored")
			result = {self.S11N_ALIAS_COMMENT_IDS: result[self.S11N_ALIAS_COMMENT_IDS]}

		if len(result) == 0:
			raise ValueError("feedback_id or comment_ids are required")

		return result


class FeedbackComment(BaseModel):
	author: FeedbackCommentAuthor
	feedback_id: int = Field(validation_alias="feedbackId")
	id: int
	status: CommentStatus
	text: str
	can_modify: Optional[bool] = Field(default=None, validation_alias="canModify")
	parent_id: Optional[int] = Field(default=None, validation_alias="parentId")


class Paging(BaseModel):
	next_page_token: Optional[str] = Field(default=None, validation_alias="nextPageToken")


class Result(BaseModel):
	comments: List[FeedbackComment]
	paging: Optional[Paging] = None


class Response(BaseResponse):
	result: Optional[Result] = None



from typing import Optional, Generic, TypeVar


T = TypeVar("T")


class Config(Generic[T]):
	__slots__ = "session", "base_url", "business_id", "campaign_id"

	session: T
	base_url: str
	business_id: Optional[int]
	campaign_id: Optional[int]

	def __init__(
		self,
		session: T,
		base_url: str,
		*,
		business_id: Optional[int] = None,
		campaign_id: Optional[int] = None,
	) -> None:
		self.session = session
		self.base_url = base_url
		self.business_id = business_id
		self.campaign_id = campaign_id



from ya_market_api.base.sync_api_mixin import SyncAPIMixin
from ya_market_api.feedback.base_api import BaseFeedbackAPI
from ya_market_api.feedback.dataclass import (
	FeedbackListRequest, FeedbackListResponse, FeedbackCommentListRequest, FeedbackCommentListResponse,
	FeedbackCommentAddRequest, FeedbackCommentAddResponse, FeedbackCommentUpdateRequest, FeedbackCommentUpdateResponse,
	FeedbackCommentDeleteRequest, FeedbackCommentDeleteResponse, FeedbackReactionSkipRequest,
	FeedbackReactionSkipResponse,
)

from typing import Optional


class SyncFeedbackAPI(SyncAPIMixin, BaseFeedbackAPI):
	def get_feedback_list(self, request: Optional[FeedbackListRequest] = None) -> FeedbackListResponse:
		request = request or FeedbackListRequest()
		url = self.router.feedback_list(self.business_id)
		response = self.session.post(
			url=url,
			params=request.model_dump_request_params(),
			json=request.model_dump_request_payload(),
		)
		self.validate_response(response)
		return FeedbackListResponse.model_validate_json(response.text)

	def get_feedback_comment_list(self, request: FeedbackCommentListRequest) -> FeedbackCommentListResponse:
		url = self.router.feedback_comment_list(self.business_id)
		response = self.session.post(
			url=url,
			params=request.model_dump_request_params(),
			json=request.model_dump_request_payload(),
		)
		self.validate_response(response)
		return FeedbackCommentListResponse.model_validate_json(response.text)

	def add_feedback_comment(self, request: FeedbackCommentAddRequest) -> FeedbackCommentAddResponse:
		url = self.router.feedback_comment_add(self.business_id)
		response = self.session.post(url=url, json=request.model_dump(by_alias=True, exclude_defaults=True))
		self.validate_response(response)
		return FeedbackCommentAddResponse.model_validate_json(response.text)

	def update_feedback_comment(self, request: FeedbackCommentUpdateRequest) -> FeedbackCommentUpdateResponse:
		url = self.router.feedback_comment_update(self.business_id)
		response = self.session.post(url=url, json=request.model_dump(by_alias=True, exclude_defaults=True))
		self.validate_response(response)
		return FeedbackCommentUpdateResponse.model_validate_json(response.text)

	def delete_feedback_comment(self, request: FeedbackCommentDeleteRequest) -> FeedbackCommentDeleteResponse:
		url = self.router.feedback_comment_delete(self.business_id)
		response = self.session.post(url=url, json=request.model_dump(by_alias=True, exclude_defaults=True))
		self.validate_response(response)
		return FeedbackCommentDeleteResponse.model_validate_json(response.text)

	def skip_feedback_reaction(self, request: FeedbackReactionSkipRequest) -> FeedbackReactionSkipResponse:
		url = self.router.feedback_reaction_skip(self.business_id)
		response = self.session.post(url=url, json=request.model_dump(by_alias=True, exclude_defaults=True))
		self.validate_response(response)
		return FeedbackReactionSkipResponse.model_validate_json(response.text)



from ya_market_api.base.async_api_mixin import AsyncAPIMixin
from ya_market_api.base.async_config import AsyncConfig
from ya_market_api.guide.base_api import BaseGuideAPI
from ya_market_api.guide.dataclass import TokenInfoResponse, DeliveryServicesResponse
from ya_market_api.guide.region.async_api import AsyncGuideRegionAPI


class AsyncGuideAPI(AsyncAPIMixin, BaseGuideAPI):
	def __init__(self, config: AsyncConfig) -> None:
		super().__init__(config)
		self.region = AsyncGuideRegionAPI(config)

	async def get_token_info(self) -> TokenInfoResponse:
		url = self.router.token_info()

		async with self.session.post(url=url, json="") as response:
			self.validate_response(response)
			return TokenInfoResponse.model_validate_json(await response.text())

	async def get_delivery_services(self) -> DeliveryServicesResponse:
		url = self.router.delivery_services()

		async with self.session.get(url=url) as response:
			self.validate_response(response)
			return DeliveryServicesResponse.model_validate_json(await response.text())



from ya_market_api.feedback.const import CommentAuthorType

from typing import Optional

from pydantic.main import BaseModel


class FeedbackCommentAuthor(BaseModel):
	name: Optional[str] = None
	type: Optional[CommentAuthorType] = None



from ya_market_api.order.dataclass.order_get import (
	OrderDeliveryDates, OrderShipment, OrderDelivery, OrderItemDetail, Order,
)

from unittest.mock import patch


class TestOrderDeliveryDates:
	def test_dates_must_be_arrow(self):
		with patch(
			"ya_market_api.order.dataclass.order_get.str_date_to_arrow",
			spec=True,
		) as str_date_to_arrow_mock:
			OrderDeliveryDates.dates_must_be_arrow("31-12-2025")
			str_date_to_arrow_mock.assert_called_once_with("31-12-2025")

	def test_optional_times_must_be_optional_time(self):
		with patch(
			"ya_market_api.order.dataclass.order_get.optional_str_time_to_optional_time",
			spec=True,
		) as optional_str_time_to_optional_time_mock:
			OrderDeliveryDates.optional_times_must_be_optional_time("12:30")
			optional_str_time_to_optional_time_mock.assert_called_once_with("12:30")

	def test_optional_dates_must_be_optional_arrow(self):
		with patch(
			"ya_market_api.order.dataclass.order_get.optional_str_date_to_optional_arrow",
			spec=True,
		) as optional_str_date_to_optional_arrow_mock:
			OrderDeliveryDates.optional_dates_must_be_optional_arrow("31-12-2025 12:30:15")
			optional_str_date_to_optional_arrow_mock.assert_called_once_with("31-12-2025 12:30:15")


class TestOrderShipment:
	def test_optional_dates_must_be_optional_arrow(self):
		with patch(
			"ya_market_api.order.dataclass.order_get.optional_str_date_to_optional_arrow",
			spec=True,
		) as optional_str_date_to_optional_arrow_mock:
			OrderShipment.optional_dates_must_be_optional_arrow("31-12-2025")
			optional_str_date_to_optional_arrow_mock.assert_called_once_with("31-12-2025")

	def test_optional_times_must_be_optional_time(self):
		with patch(
			"ya_market_api.order.dataclass.order_get.optional_str_time_to_optional_time",
			spec=True,
		) as optional_str_time_to_optional_time_mock:
			OrderShipment.optional_times_must_be_optional_time("12:30")
			optional_str_time_to_optional_time_mock.assert_called_once_with("12:30")


class TestOrderDelivery:
	def test_optional_dates_must_be_optional_arrow(self):
		with patch(
			"ya_market_api.order.dataclass.order_get.optional_str_date_to_optional_arrow",
			spec=True,
		) as optional_str_date_to_optional_arrow_mock:
			OrderDelivery.optional_dates_must_be_optional_arrow("31-12-2025")
			optional_str_date_to_optional_arrow_mock.assert_called_once_with("31-12-2025")


class TestOrderItemDetail:
	def test_dates_must_be_arrow(self):
		with patch(
			"ya_market_api.order.dataclass.order_get.str_date_to_arrow",
			spec=True,
		) as str_date_to_arrow_mock:
			OrderItemDetail.dates_must_be_arrow("31-12-2025")
			str_date_to_arrow_mock.assert_called_once_with("31-12-2025")


class TestOrder:
	def test_datetimes_must_be_arrow(self):
		with patch(
			"ya_market_api.order.dataclass.order_get.str_datetime_to_arrow",
			spec=True,
		) as str_datetime_to_arrow_mock:
			Order.datetimes_must_be_arrow("31-12-2025 12:30:15")
			str_datetime_to_arrow_mock.assert_called_once_with("31-12-2025 12:30:15")

	def test_optional_dates_must_be_optional_arrow(self):
		with patch(
			"ya_market_api.order.dataclass.order_get.optional_str_date_to_optional_arrow",
			spec=True,
		) as optional_str_date_to_optional_arrow_mock:
			Order.optional_dates_must_be_optional_arrow("31-12-2025")
			optional_str_date_to_optional_arrow_mock.assert_called_once_with("31-12-2025")

	def test_optional_datetimes_must_be_optional_arrow(self):
		with patch(
			"ya_market_api.order.dataclass.order_get.optional_str_datetime_to_optional_arrow",
			spec=True,
		) as optional_str_datetime_to_optional_arrow_mock:
			Order.optional_datetimes_must_be_optional_arrow("31-12-2025 12:30:15")
			optional_str_datetime_to_optional_arrow_mock.assert_called_once_with("31-12-2025 12:30:15")



from ya_market_api.base.enum_toolkit import allow_unknown

from enum import Enum


@allow_unknown
class Status(Enum):
	OK = "OK"
	ERROR = "ERROR"


@allow_unknown
class AuthScope(Enum):
	ALL_METHODS = "ALL_METHODS"		# полное управление кабинетом
	ALL_METHODS_READ_ONLY = "ALL_METHODS_READ_ONLY"		# просмотр всей информации в кабинете
	INVENTORY_AND_ORDER_PROCESSING = "INVENTORY_AND_ORDER_PROCESSING"		# обработка заказов и учет товаров
	INVENTORY_AND_ORDER_PROCESSING_READ_ONLY = "INVENTORY_AND_ORDER_PROCESSING_READ_ONLY"		# просмотр информации о заказах
	PRICING = "PRICING"		# управление ценами
	PRICING_READ_ONLY = "PRICING_READ_ONLY"		# просмотр цен
	OFFERS_AND_CARDS_MANAGEMENT = "OFFERS_AND_CARDS_MANAGEMENT"		# управление товарами и карточками
	OFFERS_AND_CARDS_MANAGEMENT_READ_ONLY = "OFFERS_AND_CARDS_MANAGEMENT_READ_ONLY"		# просмотр товаров и карточек
	PROMOTION = "PROMOTION"		# продвижение товаров
	PROMOTION_READ_ONLY = "PROMOTION_READ_ONLY"		# просмотр информации о продвижении товаров
	FINANCE_AND_ACCOUNTING = "FINANCE_AND_ACCOUNTING"		# просмотр финансовой информации и отчётности
	COMMUNICATION = "COMMUNICATION"		# общение с покупателями
	SETTINGS_MANAGEMENT = "SETTINGS_MANAGEMENT"		# настройка магазинов
	SUPPLIES_MANAGEMENT_READ_ONLY = "SUPPLIES_MANAGEMENT_READ_ONLY"		# получение информации по FBY-заявкам


@allow_unknown
class CurrencyType(Enum):
	RUR = "RUR"
	USD = "USD"
	EUR = "EUR"
	UAH = "UAH"
	AUD = "AUD"
	GBP = "GBP"
	BYR = "BYR"
	BYN = "BYN"
	DKK = "DKK"
	ISK = "ISK"
	KZT = "KZT"
	CAD = "CAD"
	CNY = "CNY"
	NOK = "NOK"
	XDR = "XDR"
	SGD = "SGD"
	TRY = "TRY"
	SEK = "SEK"
	CHF = "CHF"
	JPY = "JPY"
	AZN = "AZN"
	ALL = "ALL"
	DZD = "DZD"
	AOA = "AOA"
	ARS = "ARS"
	AMD = "AMD"
	AFN = "AFN"
	BHD = "BHD"
	BGN = "BGN"
	BOB = "BOB"
	BWP = "BWP"
	BND = "BND"
	BRL = "BRL"
	BIF = "BIF"
	HUF = "HUF"
	VEF = "VEF"
	KPW = "KPW"
	VND = "VND"
	GMD = "GMD"
	GHS = "GHS"
	GNF = "GNF"
	HKD = "HKD"
	GEL = "GEL"
	AED = "AED"
	EGP = "EGP"
	ZMK = "ZMK"
	ILS = "ILS"
	INR = "INR"
	IDR = "IDR"
	JOD = "JOD"
	IQD = "IQD"
	IRR = "IRR"
	YER = "YER"
	QAR = "QAR"
	KES = "KES"
	KGS = "KGS"
	COP = "COP"
	CDF = "CDF"
	CRC = "CRC"
	KWD = "KWD"
	CUP = "CUP"
	LAK = "LAK"
	LVL = "LVL"
	SLL = "SLL"
	LBP = "LBP"
	LYD = "LYD"
	SZL = "SZL"
	LTL = "LTL"
	MUR = "MUR"
	MRO = "MRO"
	MKD = "MKD"
	MWK = "MWK"
	MGA = "MGA"
	MYR = "MYR"
	MAD = "MAD"
	MXN = "MXN"
	MZN = "MZN"
	MDL = "MDL"
	MNT = "MNT"
	NPR = "NPR"
	NGN = "NGN"
	NIO = "NIO"
	NZD = "NZD"
	OMR = "OMR"
	PKR = "PKR"
	PYG = "PYG"
	PEN = "PEN"
	PLN = "PLN"
	KHR = "KHR"
	SAR = "SAR"
	RON = "RON"
	SCR = "SCR"
	SYP = "SYP"
	SKK = "SKK"
	SOS = "SOS"
	SDG = "SDG"
	SRD = "SRD"
	TJS = "TJS"
	THB = "THB"
	TWD = "TWD"
	BDT = "BDT"
	TZS = "TZS"
	TND = "TND"
	TMM = "TMM"
	UGX = "UGX"
	UZS = "UZS"
	UYU = "UYU"
	PHP = "PHP"
	DJF = "DJF"
	XAF = "XAF"
	XOF = "XOF"
	HRK = "HRK"
	CZK = "CZK"
	CLP = "CLP"
	LKR = "LKR"
	EEK = "EEK"
	ETB = "ETB"
	RSD = "RSD"
	ZAR = "ZAR"
	KRW = "KRW"
	NAD = "NAD"
	TL = "TL"
	UE = "UE"


@allow_unknown
class RegionType(Enum):
	OTHER = "OTHER"		# неизвестный регион
	CONTINENT = "CONTINENT"		# континент
	REGION = "REGION"		# регион
	COUNTRY = "COUNTRY"		# страна
	COUNTRY_DISTRICT = "COUNTRY_DISTRICT"		# область
	REPUBLIC = "REPUBLIC"		# субъект федерации
	CITY = "CITY"		# крупный город
	VILLAGE = "VILLAGE"		# город
	CITY_DISTRICT = "CITY_DISTRICT"		# район города
	SUBWAY_STATION = "SUBWAY_STATION"		# станция метро
	REPUBLIC_AREA = "REPUBLIC_AREA"		# район субъекта федерации



from ya_market_api.base.async_api_mixin import AsyncAPIMixin
from ya_market_api.guide.region.base_api import BaseGuideRegionAPI
from ya_market_api.guide.region.dataclass import (
	RegionCountriesResponse, RegionSearchRequest, RegionSearchResponse, RegionInfoRequest, RegionInfoResponse,
	RegionChildrenRequest, RegionChildrenResponse,
)


class AsyncGuideRegionAPI(AsyncAPIMixin, BaseGuideRegionAPI):
	async def get_region_countries(self) -> RegionCountriesResponse:
		url = self.router.region_countries()

		async with self.session.post(url=url, json="") as response:
			self.validate_response(response)
			return RegionCountriesResponse.model_validate_json(await response.text())

	async def search_region(self, request: RegionSearchRequest) -> RegionSearchResponse:
		url = self.router.region_search()

		async with self.session.get(
			url=url,
			params=request.model_dump(exclude_defaults=True, by_alias=True),
		) as response:
			self.validate_response(response)
			return RegionSearchResponse.model_validate_json(await response.text())

	async def get_region_info(self, request: RegionInfoRequest) -> RegionInfoResponse:
		url = self.router.region_info(request.region_id)

		async with self.session.get(url=url) as response:
			self.validate_response(response)
			return RegionInfoResponse.model_validate_json(await response.text())

	async def get_region_children(self, request: RegionChildrenRequest) -> RegionChildrenResponse:
		url = self.router.region_children(request.region_id)

		async with self.session.get(
			url=url,
			params=request.model_dump(exclude={"region_id"}, exclude_defaults=True, by_alias=True),
		) as response:
			self.validate_response(response)
			return RegionChildrenResponse.model_validate_json(await response.text())



from ya_market_api.base.const import Status

from typing import Collection

from pydantic.main import BaseModel
from pydantic.fields import Field
from pydantic.config import ConfigDict


class Request(BaseModel):
	model_config = ConfigDict(arbitrary_types_allowed=True)

	feedback_ids: Collection[int] = Field(serialization_alias="feedbackIds", min_length=1, max_length=50)


class Response(BaseModel):
	status: Status = Field(default=Status.OK)



from ya_market_api.guide.region.router import GuideRegionRouter


class TestGuideRegionRouter:
	def test_region_countries(self):
		router = GuideRegionRouter("")
		assert router.region_countries() == "/regions/countries"

	def test_region_search(self):
		router = GuideRegionRouter("")
		assert router.region_search() == "/regions"

	def test_region_info(self):
		router = GuideRegionRouter("")
		assert router.region_info(512) == "/regions/512"

	def test_region_children(self):
		router = GuideRegionRouter("")
		assert router.region_children(512) == "/regions/512/children"



from ya_market_api.base.enum_toolkit import allow_unknown

from enum import Enum


@allow_unknown
class APIAvailabilityStatus(Enum):
	AVAILABLE = "AVAILABLE"
	DISABLED_BY_INACTIVITY = "DISABLED_BY_INACTIVITY"


@allow_unknown
class PlacementType(Enum):
	FBS = "FBS"
	FBY = "FBY"
	DBS = "DBS"



from ya_market_api.guide.region.dataclass.countries import Response as RegionCountriesResponse
from ya_market_api.guide.region.dataclass.search import Request as RegionSearchRequest, Response as RegionSearchResponse
from ya_market_api.guide.region.dataclass.info import Request as RegionInfoRequest, Response as RegionInfoResponse
from ya_market_api.guide.region.dataclass.children import (
	Request as RegionChildrenRequest, Response as RegionChildrenResponse,
)


__all__ = [
	"RegionCountriesResponse", "RegionSearchRequest", "RegionSearchResponse", "RegionInfoRequest", "RegionInfoResponse",
	"RegionChildrenRequest", "RegionChildrenResponse",
]



from ya_market_api.guide.sync_api import SyncGuideAPI
from ya_market_api.guide.region.sync_api import SyncGuideRegionAPI
from ya_market_api.base.sync_config import SyncConfig

from unittest.mock import Mock, patch

from requests.sessions import Session


class TestSyncGuideAPI:
	def test___init__(self):
		session = Session()
		config = SyncConfig(session, "", business_id=1)
		api = SyncGuideAPI(config)
		assert isinstance(api.region, SyncGuideRegionAPI)
		assert api.region.session is session

	def test_get_token_info(self):
		session = Mock()
		session.post = Mock()
		session.post.return_value = Mock()
		session.post.return_value.text = "TEXT"
		config = SyncConfig(session, "", business_id=1)
		api = SyncGuideAPI(config)

		with patch("ya_market_api.guide.sync_api.TokenInfoResponse") as TokenInfoResponseMock:
			TokenInfoResponseMock.model_validate_json = Mock()
			TokenInfoResponseMock.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert api.get_token_info() == "DESERIALIZED DATA"
				TokenInfoResponseMock.model_validate_json.assert_called_once_with("TEXT")
				validate_response_mock.assert_called_once_with(session.post.return_value)
				session.post.assert_called_once_with(url=api.router.token_info(), json="")

	def test_get_delivery_services(self):
		session = Mock()
		session.get = Mock()
		session.get.return_value = Mock()
		session.get.return_value.text = "TEXT"
		config = SyncConfig(session, "", business_id=1)
		api = SyncGuideAPI(config)

		with patch("ya_market_api.guide.sync_api.DeliveryServicesResponse") as DeliveryServicesResponseMock:
			DeliveryServicesResponseMock.model_validate_json = Mock()
			DeliveryServicesResponseMock.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert api.get_delivery_services() == "DESERIALIZED DATA"
				DeliveryServicesResponseMock.model_validate_json.assert_called_once_with("TEXT")
				validate_response_mock.assert_called_once_with(session.get.return_value)
				session.get.assert_called_once_with(url=api.router.delivery_services())



from ya_market_api.base.sync_api_mixin import SyncAPIMixin
from ya_market_api.guide.region.base_api import BaseGuideRegionAPI
from ya_market_api.guide.region.dataclass import (
	RegionCountriesResponse, RegionSearchRequest, RegionSearchResponse, RegionInfoRequest, RegionInfoResponse,
	RegionChildrenRequest, RegionChildrenResponse,
)


class SyncGuideRegionAPI(SyncAPIMixin, BaseGuideRegionAPI):
	def get_region_countries(self) -> RegionCountriesResponse:
		url = self.router.region_countries()
		response = self.session.post(url=url, json="")
		self.validate_response(response)
		return RegionCountriesResponse.model_validate_json(response.text)

	def search_region(self, request: RegionSearchRequest) -> RegionSearchResponse:
		url = self.router.region_search()
		response = self.session.get(url=url, params=request.model_dump(exclude_defaults=True, by_alias=True))
		self.validate_response(response)
		return RegionSearchResponse.model_validate_json(response.text)

	def get_region_info(self, request: RegionInfoRequest) -> RegionInfoResponse:
		url = self.router.region_info(request.region_id)
		response = self.session.get(url=url)
		self.validate_response(response)
		return RegionInfoResponse.model_validate_json(response.text)

	def get_region_children(self, request: RegionChildrenRequest) -> RegionChildrenResponse:
		url = self.router.region_children(request.region_id)
		response = self.session.get(
			url=url,
			params=request.model_dump(exclude={"region_id"}, exclude_defaults=True, by_alias=True),
		)
		self.validate_response(response)
		return RegionChildrenResponse.model_validate_json(response.text)



from ya_market_api.feedback.dataclass.comment_add import (
	Request as FeedbackCommentAddRequest, Response as FeedbackCommentAddResponse,
)
from ya_market_api.feedback.dataclass.comment_delete import (
	Request as FeedbackCommentDeleteRequest, Response as FeedbackCommentDeleteResponse,
)
from ya_market_api.feedback.dataclass.comment_list import (
	Request as FeedbackCommentListRequest, Response as FeedbackCommentListResponse,
)
from ya_market_api.feedback.dataclass.comment_update import (
	Request as FeedbackCommentUpdateRequest, Response as FeedbackCommentUpdateResponse,
)
from ya_market_api.feedback.dataclass.list import (
	Request as FeedbackListRequest, Response as FeedbackListResponse,
)
from ya_market_api.feedback.dataclass.reaction_skip import (
	Request as FeedbackReactionSkipRequest, Response as FeedbackReactionSkipResponse,
)


__all__ = [
	"FeedbackCommentAddRequest", "FeedbackCommentAddResponse", "FeedbackCommentDeleteRequest",
	"FeedbackCommentDeleteResponse", "FeedbackCommentListRequest", "FeedbackCommentListResponse",
	"FeedbackCommentUpdateRequest", "FeedbackCommentUpdateResponse", "FeedbackListRequest", "FeedbackListResponse",
	"FeedbackReactionSkipRequest", "FeedbackReactionSkipResponse",
]



from ya_market_api.base.dataclass import BaseResponse
from ya_market_api.base.const import AuthScope

from typing import Set

from pydantic.main import BaseModel
from pydantic.fields import Field


class APIKey(BaseModel):
	name: str
	auth_scopes: Set[AuthScope] = Field(validation_alias="authScopes")


class Result(BaseModel):
	api_key: APIKey = Field(validation_alias="apiKey")


class Response(BaseResponse):
	result: Result



from ya_market_api.base.sync_api_mixin import SyncAPIMixin
from ya_market_api.base.sync_config import SyncConfig
from ya_market_api.guide.base_api import BaseGuideAPI
from ya_market_api.guide.dataclass import TokenInfoResponse, DeliveryServicesResponse
from ya_market_api.guide.region.sync_api import SyncGuideRegionAPI


class SyncGuideAPI(SyncAPIMixin, BaseGuideAPI):
	def __init__(self, config: SyncConfig) -> None:
		super().__init__(config)
		self.region = SyncGuideRegionAPI(config)

	def get_token_info(self) -> TokenInfoResponse:
		url = self.router.token_info()
		response = self.session.post(url=url, json="")
		self.validate_response(response)
		return TokenInfoResponse.model_validate_json(response.text)

	def get_delivery_services(self) -> DeliveryServicesResponse:
		url = self.router.delivery_services()
		response = self.session.get(url=url)
		self.validate_response(response)
		return DeliveryServicesResponse.model_validate_json(response.text)



from ya_market_api.base.enum_toolkit import allow_unknown

from enum import Enum


@allow_unknown
class CatalogLanguageType(Enum):
	RU = "RU"		# русский
	UZ = "UZ"		# узбекский


@allow_unknown
class OfferCardStatusType(Enum):
	HAS_CARD_CAN_NOT_UPDATE = "HAS_CARD_CAN_NOT_UPDATE"		# Карточка Маркета.
	HAS_CARD_CAN_UPDATE = "HAS_CARD_CAN_UPDATE"		# Можно дополнить.
	HAS_CARD_CAN_UPDATE_ERRORS = "HAS_CARD_CAN_UPDATE_ERRORS"		# Изменения не приняты.
	HAS_CARD_CAN_UPDATE_PROCESSING = "HAS_CARD_CAN_UPDATE_PROCESSING"		# Изменения на проверке.
	NO_CARD_NEED_CONTENT = "NO_CARD_NEED_CONTENT"		# Создайте карточку.
	NO_CARD_MARKET_WILL_CREATE = "NO_CARD_MARKET_WILL_CREATE"		# Создаст Маркет.
	NO_CARD_ERRORS = "NO_CARD_ERRORS"		# Не создана из-за ошибки.
	NO_CARD_PROCESSING = "NO_CARD_PROCESSING"		# Проверяем данные.
	NO_CARD_ADD_TO_CAMPAIGN = "NO_CARD_ADD_TO_CAMPAIGN"		# Разместите товар в магазине.


@allow_unknown
class AgeUnit(Enum):
	YEAR = "YEAR"
	MONTH = "MONTH"


@allow_unknown
class CampaignStatusType(Enum):
	PUBLISHED = "PUBLISHED"		# Готов к продаже
	CHECKING = "CHECKING"		# На проверке
	DISABLED_BY_PARTNER = "DISABLED_BY_PARTNER"		# Скрыт вами
	DISABLED_AUTOMATICALLY = "DISABLED_AUTOMATICALLY"		# Отклонен
	REJECTED_BY_MARKET = "REJECTED_BY_MARKET"		# Исправьте ошибки
	CREATING_CARD = "CREATING_CARD"		# Создается карточка
	NO_CARD = "NO_CARD"		# Нужна карточка
	NO_STOCKS = "NO_STOCKS"		# Нет на складе
	ARCHIVED = "ARCHIVED"		# В архиве


@allow_unknown
class CommodityCodeType(Enum):
	CUSTOMS_COMMODITY_CODE = "CUSTOMS_COMMODITY_CODE"
	IKPU_CODE = "IKPU_CODE"


@allow_unknown
class OfferConditionQualityType(Enum):
	PERFECT = "PERFECT"		# идеальный
	EXCELLENT = "EXCELLENT"		# отличный
	GOOD = "GOOD"		# хороший
	NOT_SPECIFIED = "NOT_SPECIFIED"		# не выбран


@allow_unknown
class OfferConditionType(Enum):
	PREOWNED = "PREOWNED"		# бывший в употреблении товар, раньше принадлежал другому человеку
	SHOWCASESAMPLE = "SHOWCASESAMPLE"		# витринный образец
	REFURBISHED = "REFURBISHED"		# повторная продажа товара
	REDUCTION = "REDUCTION"		# товар с дефектами
	RENOVATED = "RENOVATED"		# восстановленный товар
	NOT_SPECIFIED = "NOT_SPECIFIED"		# не выбран


@allow_unknown
class TimeUnit(Enum):
	HOUR = "HOUR"		# час
	DAY = "DAY"		# сутки
	WEEK = "WEEK"		# неделя
	MONTH = "MONTH"		# месяц
	YEAR = "YEAR"		# год


@allow_unknown
class MediaFileUploadState(Enum):
	UPLOADING = "UPLOADING"		# загружается
	UPLOADED = "UPLOADED"		# успешно загружен
	FAILED = "FAILED"		# при загрузке произошла ошибка. Повторите попытку позже


@allow_unknown
class SellingProgramType(Enum):
	FBY = "FBY"		# FBY
	FBS = "FBS"		# FBS
	DBS = "DBS"		# DBS
	EXPRESS = "EXPRESS"		# Экспресс


@allow_unknown
class SellingProgramStatusType(Enum):
	FINE = "FINE"		# доступно
	REJECT = "REJECT"		# недоступно


@allow_unknown
class OfferType(Enum):
	DEFAULT = "DEFAULT"		# товары, для которых вы передавали особый тип ранее и хотите убрать его
	MEDICINE = "MEDICINE"		# лекарства
	BOOK = "BOOK"		# бумажные и электронные книги
	AUDIOBOOK = "AUDIOBOOK"		# аудиокниги
	ARTIST_TITLE = "ARTIST_TITLE"		# музыкальная и видеопродукция
	ON_DEMAND = "ON_DEMAND"		# товары на заказ
	ALCOHOL = "ALCOHOL"		# алкоголь


@allow_unknown
class ShowcaseType(Enum):
	B2B = "B2B"
	B2C = "B2C"



from ya_market_api.base.api import API
from ya_market_api.exception import CampaignIdError
from ya_market_api.order.router import OrderRouter


class BaseOrderAPI(API[OrderRouter]):
	@property
	def campaign_id(self) -> int:
		if self.config.campaign_id is None:
			raise CampaignIdError("The campaign_id was not specified")

		return self.config.campaign_id

	@staticmethod
	def make_router(base_url: str) -> OrderRouter:
		return OrderRouter(base_url)



from ya_market_api.base.dataclass.region import Region

from typing import Optional, List

from pydantic.main import BaseModel
from pydantic.fields import Field


class Request(BaseModel):
	name: str
	limit: Optional[int] = None
	page_token: Optional[str] = None


class ResponsePaging(BaseModel):
	next_page_token: Optional[str] = Field(default=None, validation_alias="nextPageToken")


class Response(BaseModel):
	regions: List[Region]
	paging: Optional[ResponsePaging] = None



from ya_market_api.base.async_api_mixin import AsyncAPIMixin
from ya_market_api.base.async_config import AsyncConfig
from ya_market_api.exception import InvalidResponseError, AuthorizationError, NotFoundError

from unittest.mock import Mock
from http import HTTPStatus
from typing import Optional, Type

import pytest
from aiohttp.client import ClientSession


class BaseClient:
	def __init__(self, *args, **kwargs) -> None:
		pass


class Client(AsyncAPIMixin, BaseClient):
	pass


class TestAsyncAPIMixin:
	@pytest.mark.asyncio()
	@pytest.mark.parametrize(
		"status, expected_error_type, expected_error_text",
		[
			(HTTPStatus.OK, None, None),
			(HTTPStatus.FORBIDDEN, AuthorizationError, "Unauthorized"),
			(HTTPStatus.UNAUTHORIZED, AuthorizationError, "Unauthorized"),
			(HTTPStatus.NOT_FOUND, NotFoundError, "Resource was not found"),
			(HTTPStatus.INTERNAL_SERVER_ERROR, InvalidResponseError, "Response is not valid"),
		],
	)
	async def test_validate_response(
		self,
		status: HTTPStatus,
		expected_error_type: Optional[Type[Exception]],
		expected_error_text: Optional[str],
	):
		session = ClientSession()
		config = AsyncConfig(session, "", business_id=1)
		api = Client(config)
		response = Mock()
		response.status = status
		response.ok = status < 400

		if expected_error_type is None:
			assert api.validate_response(response) is None
		else:
			with pytest.raises(expected_error_type, match=expected_error_text):
				api.validate_response(response)



from tests.fake_async_session import FakeAsyncSession
from ya_market_api.feedback.async_api import AsyncFeedbackAPI
from ya_market_api.feedback.dataclass import (
	FeedbackListRequest, FeedbackCommentListRequest, FeedbackCommentAddRequest, FeedbackCommentUpdateRequest,
	FeedbackCommentDeleteRequest, FeedbackReactionSkipRequest,
)
from ya_market_api.base.async_config import AsyncConfig

from unittest.mock import patch, Mock

import pytest


class TestAsyncFeedbackAPI:
	@pytest.mark.asyncio()
	async def test_get_feedback_list(self):
		session = FakeAsyncSession("RAW DATA")
		config = AsyncConfig(session, "", business_id=1)		# type: ignore - for testing purposes
		api = AsyncFeedbackAPI(config)
		request = FeedbackListRequest(limit=50, page_token="page-token", feedback_ids=(1, 2, 3))

		with patch("ya_market_api.feedback.async_api.FeedbackListResponse") as FeedbackListResponseMock:
			FeedbackListResponseMock.model_validate_json = Mock()
			FeedbackListResponseMock.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert await api.get_feedback_list() == "DESERIALIZED DATA"
				FeedbackListResponseMock.model_validate_json.assert_called_once_with("RAW DATA")
				validate_response_mock.assert_called_once_with(session.response)
				assert session.last_call_method == "POST"
				assert session.last_call_url == api.router.feedback_list(1)
				assert session.last_call_json == {}
				assert session.last_call_params == {}

				assert await api.get_feedback_list(request) == "DESERIALIZED DATA"
				assert session.last_call_method == "POST"
				assert session.last_call_url == api.router.feedback_list(1)
				assert session.last_call_json == {"feedbackIds": (1, 2, 3)}
				assert session.last_call_params == {"limit": 50, "page_token": "page-token"}

	@pytest.mark.asyncio()
	async def test_get_feedback_comment_list(self):
		session = FakeAsyncSession("RAW DATA")
		config = AsyncConfig(session, "", business_id=1)		# type: ignore - for testing purposes
		api = AsyncFeedbackAPI(config)
		request = FeedbackCommentListRequest(limit=50, page_token="page-token", comment_ids=(1, 2, 3))

		with patch("ya_market_api.feedback.async_api.FeedbackCommentListResponse") as FeedbackCommentListResponseMock:
			FeedbackCommentListResponseMock.model_validate_json = Mock()
			FeedbackCommentListResponseMock.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert await api.get_feedback_comment_list(request) == "DESERIALIZED DATA"
				FeedbackCommentListResponseMock.model_validate_json.assert_called_once_with("RAW DATA")
				validate_response_mock.assert_called_once_with(session.response)
				assert session.last_call_method == "POST"
				assert session.last_call_url == api.router.feedback_comment_list(1)
				assert session.last_call_json == {"commentIds": (1, 2, 3)}
				assert session.last_call_params == {"limit": 50, "page_token": "page-token"}

	@pytest.mark.asyncio()
	async def test_add_feedback_comment(self):
		session = FakeAsyncSession("RAW DATA")
		config = AsyncConfig(session, "", business_id=1)		# type: ignore - for testing purposes
		api = AsyncFeedbackAPI(config)
		request = FeedbackCommentAddRequest.create(512, "COMMENT", 1024)

		with patch("ya_market_api.feedback.async_api.FeedbackCommentAddResponse") as FeedbackCommentAddResponse:
			FeedbackCommentAddResponse.model_validate_json = Mock()
			FeedbackCommentAddResponse.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert await api.add_feedback_comment(request) == "DESERIALIZED DATA"
				FeedbackCommentAddResponse.model_validate_json.assert_called_once_with("RAW DATA")
				validate_response_mock.assert_called_once_with(session.response)
				assert session.last_call_method == "POST"
				assert session.last_call_url == api.router.feedback_comment_add(1)
				assert session.last_call_json == {"feedbackId": 512, "comment": {"parentId": 1024, "text": "COMMENT"}}

	@pytest.mark.asyncio()
	async def test_update_feedback_comment(self):
		session = FakeAsyncSession("RAW DATA")
		config = AsyncConfig(session, "", business_id=1)		# type: ignore - for testing purposes
		api = AsyncFeedbackAPI(config)
		request = FeedbackCommentUpdateRequest.create(512, 1024, "COMMENT")

		with patch("ya_market_api.feedback.async_api.FeedbackCommentUpdateResponse") as FeedbackCommentUpdateResponse:
			FeedbackCommentUpdateResponse.model_validate_json = Mock()
			FeedbackCommentUpdateResponse.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert await api.update_feedback_comment(request) == "DESERIALIZED DATA"
				FeedbackCommentUpdateResponse.model_validate_json.assert_called_once_with("RAW DATA")
				validate_response_mock.assert_called_once_with(session.response)
				assert session.last_call_method == "POST"
				assert session.last_call_url == api.router.feedback_comment_update(1)
				assert session.last_call_json == {"feedbackId": 512, "comment": {"id": 1024, "text": "COMMENT"}}

	@pytest.mark.asyncio()
	async def test_delete_feedback_comment(self):
		session = FakeAsyncSession("RAW DATA")
		config = AsyncConfig(session, "", business_id=1)		# type: ignore - for testing purposes
		api = AsyncFeedbackAPI(config)
		request = FeedbackCommentDeleteRequest(id=512)

		with patch("ya_market_api.feedback.async_api.FeedbackCommentDeleteResponse") as FeedbackCommentDeleteResponse:
			FeedbackCommentDeleteResponse.model_validate_json = Mock()
			FeedbackCommentDeleteResponse.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert await api.delete_feedback_comment(request) == "DESERIALIZED DATA"
				FeedbackCommentDeleteResponse.model_validate_json.assert_called_once_with("RAW DATA")
				validate_response_mock.assert_called_once_with(session.response)
				assert session.last_call_method == "POST"
				assert session.last_call_url == api.router.feedback_comment_delete(1)
				assert session.last_call_json == {"id": 512}

	@pytest.mark.asyncio()
	async def test_skip_feedback_reaction(self):
		session = FakeAsyncSession("RAW DATA")
		config = AsyncConfig(session, "", business_id=1)		# type: ignore - for testing purposes
		api = AsyncFeedbackAPI(config)
		request = FeedbackReactionSkipRequest(feedback_ids=(1, 2, 3))

		with patch("ya_market_api.feedback.async_api.FeedbackReactionSkipResponse") as FeedbackReactionSkipResponse:
			FeedbackReactionSkipResponse.model_validate_json = Mock()
			FeedbackReactionSkipResponse.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert await api.skip_feedback_reaction(request) == "DESERIALIZED DATA"
				FeedbackReactionSkipResponse.model_validate_json.assert_called_once_with("RAW DATA")
				validate_response_mock.assert_called_once_with(session.response)
				assert session.last_call_method == "POST"
				assert session.last_call_url == api.router.feedback_reaction_skip(1)
				assert session.last_call_json == {"feedbackIds": (1, 2, 3)}



from ya_market_api.base.dataclass.base_response import BaseResponse
from ya_market_api.base.dataclass.flipping_pager import FlippingPager
from ya_market_api.base.dataclass.region import Region
from ya_market_api.base.dataclass.gps import GPS


__all__ = ["BaseResponse", "FlippingPager", "Region", "GPS"]



from ya_market_api.offer.base_api import BaseOfferAPI
from ya_market_api.exception import BusinessIdError
from ya_market_api.base.config import Config

import pytest


class TestBaseOfferAPI:
	def test_business_id(self):
		config = Config(None, "")
		api = BaseOfferAPI(config)

		with pytest.raises(BusinessIdError, match="The business_id was not specified"):
			api.business_id

		config.business_id = 512
		assert api.business_id == 512



from ya_market_api.feedback.sync_api import SyncFeedbackAPI
from ya_market_api.feedback.dataclass import (
	FeedbackListRequest, FeedbackCommentListRequest, FeedbackCommentAddRequest, FeedbackCommentUpdateRequest,
	FeedbackCommentDeleteRequest, FeedbackReactionSkipRequest,
)
from ya_market_api.base.sync_config import SyncConfig

from unittest.mock import patch, Mock


class TestSyncFeedbackAPI:
	def test_get_feedback_list(self):
		session = Mock()
		session.post = Mock()
		session.post.return_value = Mock()
		session.post.return_value.text = "RAW DATA"
		config = SyncConfig(session, "", business_id=1)
		api = SyncFeedbackAPI(config)
		request = FeedbackListRequest(limit=50, page_token="page-token", feedback_ids=(1, 2, 3))

		with patch("ya_market_api.feedback.sync_api.FeedbackListResponse") as FeedbackListResponseMock:
			FeedbackListResponseMock.model_validate_json = Mock()
			FeedbackListResponseMock.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert api.get_feedback_list() == "DESERIALIZED DATA"
				FeedbackListResponseMock.model_validate_json.assert_called_once_with("RAW DATA")
				validate_response_mock.assert_called_once_with(session.post.return_value)
				session.post.assert_called_once_with(url=api.router.feedback_list(1), params={}, json={})

				session.post.reset_mock()
				assert api.get_feedback_list(request) == "DESERIALIZED DATA"
				session.post.assert_called_once_with(
					url=api.router.feedback_list(1),
					params={"limit": 50, "page_token": "page-token"},
					json={"feedbackIds": (1, 2, 3)},
				)

	def test_get_feedback_comment_list(self):
		session = Mock()
		session.post = Mock()
		session.post.return_value = Mock()
		session.post.return_value.text = "RAW DATA"
		config = SyncConfig(session, "", business_id=1)
		api = SyncFeedbackAPI(config)
		request = FeedbackCommentListRequest(comment_ids=(1, 2, 3), limit=50, page_token="page-token")

		with patch("ya_market_api.feedback.sync_api.FeedbackCommentListResponse") as FeedbackCommentListResponseMock:
			FeedbackCommentListResponseMock.model_validate_json = Mock()
			FeedbackCommentListResponseMock.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert api.get_feedback_comment_list(request) == "DESERIALIZED DATA"
				FeedbackCommentListResponseMock.model_validate_json.assert_called_once_with("RAW DATA")
				validate_response_mock.assert_called_once_with(session.post.return_value)
				session.post.assert_called_once_with(
					url=api.router.feedback_comment_list(1),
					params={"limit": 50, "page_token": "page-token"},
					json={"commentIds": (1, 2, 3)},
				)

	def test_add_feedback_comment(self):
		session = Mock()
		session.post = Mock()
		session.post.return_value = Mock()
		session.post.return_value.text = "RAW DATA"
		config = SyncConfig(session, "", business_id=1)
		api = SyncFeedbackAPI(config)
		request = FeedbackCommentAddRequest.create(512, "COMMENT", 1024)

		with patch("ya_market_api.feedback.sync_api.FeedbackCommentAddResponse") as FeedbackCommentAddResponseMock:
			FeedbackCommentAddResponseMock.model_validate_json = Mock()
			FeedbackCommentAddResponseMock.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert api.add_feedback_comment(request) == "DESERIALIZED DATA"
				FeedbackCommentAddResponseMock.model_validate_json.assert_called_once_with("RAW DATA")
				validate_response_mock.assert_called_once_with(session.post.return_value)
				session.post.assert_called_once_with(
					url=api.router.feedback_comment_add(1),
					json={"feedbackId": 512, "comment": {"parentId": 1024, "text": "COMMENT"}},
				)

	def test_update_feedback_comment(self):
		session = Mock()
		session.post = Mock()
		session.post.return_value = Mock()
		session.post.return_value.text = "RAW DATA"
		config = SyncConfig(session, "", business_id=1)
		api = SyncFeedbackAPI(config)
		request = FeedbackCommentUpdateRequest.create(512, 1024, "COMMENT")

		with patch("ya_market_api.feedback.sync_api.FeedbackCommentUpdateResponse") as FeedbackCommentUpdateResponseMock:
			FeedbackCommentUpdateResponseMock.model_validate_json = Mock()
			FeedbackCommentUpdateResponseMock.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert api.update_feedback_comment(request) == "DESERIALIZED DATA"
				FeedbackCommentUpdateResponseMock.model_validate_json.assert_called_once_with("RAW DATA")
				validate_response_mock.assert_called_once_with(session.post.return_value)
				session.post.assert_called_once_with(
					url=api.router.feedback_comment_update(1),
					json={"feedbackId": 512, "comment": {"id": 1024, "text": "COMMENT"}},
				)

	def test_delete_feedback_comment(self):
		session = Mock()
		session.post = Mock()
		session.post.return_value = Mock()
		session.post.return_value.text = "RAW DATA"
		config = SyncConfig(session, "", business_id=1)
		api = SyncFeedbackAPI(config)
		request = FeedbackCommentDeleteRequest(id=512)

		with patch("ya_market_api.feedback.sync_api.FeedbackCommentDeleteResponse") as FeedbackCommentDeleteResponseMock:
			FeedbackCommentDeleteResponseMock.model_validate_json = Mock()
			FeedbackCommentDeleteResponseMock.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert api.delete_feedback_comment(request) == "DESERIALIZED DATA"
				FeedbackCommentDeleteResponseMock.model_validate_json.assert_called_once_with("RAW DATA")
				validate_response_mock.assert_called_once_with(session.post.return_value)
				session.post.assert_called_once_with(
					url=api.router.feedback_comment_delete(1),
					json={"id": 512},
				)

	def test_skip_feedback_reaction(self):
		session = Mock()
		session.post = Mock()
		session.post.return_value = Mock()
		session.post.return_value.text = "RAW DATA"
		config = SyncConfig(session, "", business_id=1)
		api = SyncFeedbackAPI(config)
		request = FeedbackReactionSkipRequest(feedback_ids=(1, 2, 3))

		with patch("ya_market_api.feedback.sync_api.FeedbackReactionSkipResponse") as FeedbackReactionSkipResponseMock:
			FeedbackReactionSkipResponseMock.model_validate_json = Mock()
			FeedbackReactionSkipResponseMock.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert api.skip_feedback_reaction(request) == "DESERIALIZED DATA"
				FeedbackReactionSkipResponseMock.model_validate_json.assert_called_once_with("RAW DATA")
				validate_response_mock.assert_called_once_with(session.post.return_value)
				session.post.assert_called_once_with(
					url=api.router.feedback_reaction_skip(1),
					json={"feedbackIds": (1, 2, 3)},
				)



from ya_market_api.generic.functools import cache
from ya_market_api.base.router import Router


class GuideRouter(Router):
	@cache
	def token_info(self) -> str:
		return f"{self.base_url}/auth/token"

	@cache
	def delivery_services(self) -> str:
		return f"{self.base_url}/delivery/services"



from ya_market_api.campaign.dataclass.campaign_list import Request

import pytest


class TestRequest:
	def test_page_is_valid(self):
		with pytest.raises(ValueError, match="The page cannot be less than 1 or greater than 10000"):
			Request.page_is_valid(0)

		with pytest.raises(ValueError, match="The page cannot be less than 1 or greater than 10000"):
			Request.page_is_valid(10_001)

		assert Request.page_is_valid(None) is None
		assert Request.page_is_valid(512) == 512




from ya_market_api.base.const import Status

from pydantic.main import BaseModel
from pydantic.fields import Field


class BaseResponse(BaseModel):
	status: Status = Field(default=Status.OK)
	result: BaseModel



from typing import Optional
from datetime import time

from arrow import Arrow, get as get_arrow


def str_date_to_arrow(value: str, format: str = "DD-MM-YYYY") -> Arrow:
	return get_arrow(value, format)


def optional_str_date_to_optional_arrow(value: Optional[str], format: str = "DD-MM-YYYY") -> Optional[Arrow]:
	if value is None:
		return None

	return str_date_to_arrow(value, format)


def str_time_to_time(value: str) -> time:
	if not isinstance(value, str):
		raise ValueError("Time must be of string type")

	time_fragments = [int(i) for i in value.split(":")[:2]]

	if len(time_fragments) != 2:
		raise ValueError("Time must have hours and minutes")

	return time(hour=time_fragments[0], minute=time_fragments[1])


def optional_str_time_to_optional_time(value: Optional[str]) -> Optional[time]:
	if value is None:
		return None

	return str_time_to_time(value)


def str_datetime_to_arrow(value: str, format: str = "DD-MM-YYYY hh:mm:ss", tzinfo: str = "Europe/Moscow") -> Arrow:
	return get_arrow(value, format, tzinfo=tzinfo)


def optional_str_datetime_to_optional_arrow(
	value: Optional[str],
	format: str = "DD-MM-YYYY hh:mm:ss",
	tzinfo: str = "Europe/Moscow",
) -> Optional[Arrow]:
	if value is None:
		return None

	return str_datetime_to_arrow(value, format, tzinfo)



from ya_market_api.offer.dataclass.offer_list_by_business import (
	Request as OfferListByBusinessRequest, Response as OfferListByBusinessResponse,
)


__all__ = ["OfferListByBusinessRequest", "OfferListByBusinessResponse"]



from ya_market_api.base.convert import (
	str_date_to_arrow, optional_str_date_to_optional_arrow, str_time_to_time, optional_str_time_to_optional_time,
	str_datetime_to_arrow, optional_str_datetime_to_optional_arrow,
)

from datetime import time

import pytest
from arrow import get as get_arrow


def test_str_date_to_arrow():
	assert str_date_to_arrow("31-12-2025") == get_arrow(2025, 12, 31)
	assert str_date_to_arrow("2025-12-31", "YYYY-MM-DD") == get_arrow(2025, 12, 31)


def test_optional_str_date_to_optional_arrow():
	assert optional_str_date_to_optional_arrow(None) is None
	assert optional_str_date_to_optional_arrow(None, "YYYY-MM-DD") is None
	assert optional_str_date_to_optional_arrow("31-12-2025") == get_arrow(2025, 12, 31)
	assert optional_str_date_to_optional_arrow("2025-12-31", "YYYY-MM-DD") == get_arrow(2025, 12, 31)


def test_str_time_to_time():
	assert str_time_to_time("12:30") == time(hour=12, minute=30)
	assert str_time_to_time("12:30:15") == time(hour=12, minute=30)

	with pytest.raises(ValueError, match="Time must be of string type"):
		str_time_to_time(None)		# type: ignore

	with pytest.raises(ValueError, match="Time must have hours and minutes"):
		str_time_to_time("12")		# type: ignore


def test_optional_str_time_to_optional_time():
	assert optional_str_time_to_optional_time(None) is None
	assert optional_str_time_to_optional_time("12:30") == time(hour=12, minute=30)


def test_str_datetime_to_arrow():
	assert str_datetime_to_arrow("31-12-2025 12:30:15") == get_arrow(2025, 12, 31, 12, 30, 15, tzinfo="Europe/Moscow")
	assert (
		str_datetime_to_arrow(
			"2025-12-31T12:30:15",
			"YYYY-MM-DDThh:mm:ss",
		) == get_arrow(
			2025, 12, 31, 12, 30, 15, tzinfo="Europe/Moscow",
		)
	)
	assert (
		str_datetime_to_arrow(
			"2025-12-31T12:30:15",
			"YYYY-MM-DDThh:mm:ss",
			"Asia/Hong_Kong",
		) == get_arrow(
			2025, 12, 31, 12, 30, 15, tzinfo="Asia/Hong_Kong",
		)
	)


def test_optional_str_datetime_to_optional_arrow():
	assert optional_str_datetime_to_optional_arrow(None) is None
	assert (
		optional_str_datetime_to_optional_arrow(
			"31-12-2025 12:30:15"
		) == get_arrow(
			2025, 12, 31, 12, 30, 15, tzinfo="Europe/Moscow",
		)
	)
	assert (
		optional_str_datetime_to_optional_arrow(
			"2025-12-31T12:30:15",
			"YYYY-MM-DDThh:mm:ss",
		) == get_arrow(
			2025, 12, 31, 12, 30, 15, tzinfo="Europe/Moscow",
		)
	)
	assert (
		optional_str_datetime_to_optional_arrow(
			"2025-12-31T12:30:15",
			"YYYY-MM-DDThh:mm:ss",
			"Asia/Hong_Kong",
		) == get_arrow(
			2025, 12, 31, 12, 30, 15, tzinfo="Asia/Hong_Kong",
		)
	)



from ya_market_api.base.async_api_mixin import AsyncAPIMixin
from ya_market_api.campaign.base_api import BaseCampaignAPI
from ya_market_api.campaign.dataclass import CampaignListRequest, CampaignListResponse

from typing import Optional


class AsyncCampaignAPI(AsyncAPIMixin, BaseCampaignAPI):
	async def get_campaign_list(self, request: Optional[CampaignListRequest] = None) -> CampaignListResponse:
		request = request or CampaignListRequest()
		url = self.router.campaign_list()
		async with self.session.get(
			url=url,
			params=request.model_dump(by_alias=True, exclude_none=True),
		) as response:
			self.validate_response(response)
			return CampaignListResponse.model_validate_json(await response.text())



from ya_market_api.base.api import API
from ya_market_api.guide.region.router import GuideRegionRouter


class BaseGuideRegionAPI(API[GuideRegionRouter]):
	@staticmethod
	def make_router(base_url: str):
		return GuideRegionRouter(base_url)



from ya_market_api.base.router import Router


class FeedbackRouter(Router):
	def feedback_list(self, business_id: int) -> str:
		return f"{self.base_url}/businesses/{business_id}/goods-feedback"

	def feedback_comment_list(self, business_id: int) -> str:
		return f"{self.base_url}/businesses/{business_id}/goods-feedback/comments"

	def feedback_comment_add(self, business_id: int) -> str:
		return f"{self.base_url}/businesses/{business_id}/goods-feedback/comments/update"

	def feedback_comment_update(self, business_id: int) -> str:
		return f"{self.base_url}/businesses/{business_id}/goods-feedback/comments/update"

	def feedback_comment_delete(self, business_id: int) -> str:
		return f"{self.base_url}/businesses/{business_id}/goods-feedback/comments/delete"

	def feedback_reaction_skip(self, business_id: int) -> str:
		return f"{self.base_url}/businesses/{business_id}/goods-feedback/skip-reaction"



from ya_market_api.offer.dataclass.offer_list_by_business import Request, Price, PriceWithDiscount
from ya_market_api.offer.const import OfferCardStatusType, CatalogLanguageType

import pytest
from arrow import get as get_arrow


class TestRequest:
	def test_limit_must_be_greater_than_1(self):
		assert Request.limit_must_be_greater_than_0(None) is None
		assert Request.limit_must_be_greater_than_0(10) == 10

		with pytest.raises(ValueError, match="Limit cannot be less than 1"):
			Request.limit_must_be_greater_than_0(0)

	def test_card_statuses_must_be_filled(self):
		assert Request.card_statuses_must_be_filled(None) is None
		assert (
			Request.card_statuses_must_be_filled([OfferCardStatusType.HAS_CARD_CAN_NOT_UPDATE]) == [OfferCardStatusType.HAS_CARD_CAN_NOT_UPDATE]
		)

		with pytest.raises(ValueError, match="Card statuses length cannot be less than 1"):
			Request.card_statuses_must_be_filled([])

	def test_category_ids_must_be_filled(self):
		assert Request.category_ids_must_be_filled(None) is None
		assert Request.category_ids_must_be_filled([1, 2, 3]) == [1, 2, 3]

		with pytest.raises(ValueError, match="Category ids length cannot be less than 1"):
			assert Request.category_ids_must_be_filled([])

	def test_offer_ids_must_be_filled(self):
		assert Request.offer_ids_must_be_filled(None) is None
		assert Request.offer_ids_must_be_filled(["1", "2", "3"]) == ["1", "2", "3"]

		with pytest.raises(ValueError, match="Offer ids length cannot be less than 1"):
			Request.offer_ids_must_be_filled([])

		with pytest.raises(ValueError, match="Offer ids length cannot be greater than 200"):
			Request.offer_ids_must_be_filled(list(map(str, range(201))))

		with pytest.raises(ValueError, match="Offer id length cannot be less than 1"):
			Request.offer_ids_must_be_filled([""])

		with pytest.raises(ValueError, match="Offer id length cannot be greater than 250"):
			Request.offer_ids_must_be_filled(["".join(map(str, range(251)))])

	def test_tags_must_be_filled(self):
		assert Request.tags_must_be_filled(None) is None
		assert Request.tags_must_be_filled(["tag"]) == ["tag"]

		with pytest.raises(ValueError, match="Tags length cannot be less than 1"):
			Request.tags_must_be_filled([])

	def test_vendor_names_must_be_filled(self):
		assert Request.vendor_names_must_be_filled(None) is None
		assert Request.vendor_names_must_be_filled(["vendor"]) == ["vendor"]

		with pytest.raises(ValueError, match="Vendor names length cannot be less than 1"):
			Request.vendor_names_must_be_filled([])

	def test_serialize_language(self):
		request = Request()
		assert request.serialize_language(None) is None
		assert request.serialize_language(CatalogLanguageType.RU) == "RU"

	def test_serialize_card_statuses(self):
		request = Request()
		assert request.serialize_card_statuses(None) is None
		assert request.serialize_card_statuses([]) == []
		assert (
			request.serialize_card_statuses([OfferCardStatusType.HAS_CARD_CAN_UPDATE]) == [OfferCardStatusType.HAS_CARD_CAN_UPDATE.value]
		)

	def test_model_dump_request_params(self):
		request = Request()
		assert request.model_dump_request_params() == {}

		request.limit = 20
		request.page_token = "page-token"
		assert request.model_dump_request_params() == {"limit": 20, "page_token": "page-token"}

		request.offer_ids = ["1", "2", "3"]

		with pytest.warns(UserWarning, match="When using offer_ids, the other query parameters will be ignored"):
			assert request.model_dump_request_params() == {}

	def test_model_dump_request_payload(self):
		request = Request()
		assert request.model_dump_request_payload() == {}

		request.category_ids = [1, 2, 3]
		request.archived = True
		assert request.model_dump_request_payload() == {"categoryIds": [1, 2, 3], "archived": True}

		request.offer_ids = ["1", "2", "3"]

		with pytest.warns(UserWarning, match="When using offer_ids, the other parameters will be ignored"):
			assert request.model_dump_request_payload() == {"offerIds": ["1", "2", "3"]}


class TestPrice:
	def test_datetimes_must_be_arrow(self):
		assert Price.datetimes_must_be_arrow("2025-01-01T00:00:00") == get_arrow(2025, 1, 1)


class TestBasicPrice:
	def test_datetimes_must_be_arrow(self):
		assert PriceWithDiscount.datetimes_must_be_arrow("2025-01-01T00:00:00") == get_arrow(2025, 1, 1)



from ya_market_api.base.dataclass import FlippingPager, Region

from typing import Optional

from pydantic.main import BaseModel
from pydantic.fields import Field
from pydantic.functional_validators import field_validator


class Request(BaseModel):
	region_id: int
	page: Optional[int] = None
	page_size: Optional[int] = Field(default=None, serialization_alias="pageSize")

	@field_validator("page", mode="after")
	@classmethod
	def page_is_valid(cls, value: Optional[int]) -> Optional[int]:
		if value is None:
			return None

		if value < 1 or value > 10_000:
			raise ValueError("The page cannot be less than 1 or greater than 10000")

		return value

	@field_validator("page_size", mode="after")
	@classmethod
	def page_size_is_valid(cls, value: Optional[int]) -> Optional[int]:
		if value is None:
			return None

		if value < 1:
			raise ValueError("The page_size cannot be less than 1")

		return value


class Response(BaseModel):
	pager: Optional[FlippingPager] = None
	region: Optional[Region] = Field(default=None, validation_alias="regions")



from tests.fake_async_session import FakeAsyncSession
from ya_market_api.guide.async_api import AsyncGuideAPI
from ya_market_api.guide.region.async_api import AsyncGuideRegionAPI
from ya_market_api.base.async_config import AsyncConfig

from unittest.mock import patch, Mock

import pytest
from aiohttp.client import ClientSession


class TestAsyncGuideAPI:
	@pytest.mark.asyncio()
	async def test___init__(self):
		session = ClientSession()
		config = AsyncConfig(session, "", business_id=1)
		api = AsyncGuideAPI(config)
		assert isinstance(api.region, AsyncGuideRegionAPI)
		assert api.region.session is session

	@pytest.mark.asyncio()
	async def test_get_token_info(self):
		session = FakeAsyncSession("RAW DATA")
		config = AsyncConfig(session, "", business_id=1)				# type: ignore - for testing purposes
		api = AsyncGuideAPI(config)

		with patch("ya_market_api.guide.async_api.TokenInfoResponse") as TokenInfoResponseMock:
			TokenInfoResponseMock.model_validate_json = Mock()
			TokenInfoResponseMock.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert await api.get_token_info() == "DESERIALIZED DATA"
				TokenInfoResponseMock.model_validate_json.assert_called_once_with("RAW DATA")
				validate_response_mock.assert_called_once_with(session.response)
				assert session.last_call_method == "POST"
				assert session.last_call_url == api.router.token_info()
				assert session.last_call_json == ""

	@pytest.mark.asyncio()
	async def test_get_delivery_services(self):
		session = FakeAsyncSession("RAW DATA")
		config = AsyncConfig(session, "", business_id=1)		# type: ignore - for testing purposes
		api = AsyncGuideAPI(config)

		with patch("ya_market_api.guide.async_api.DeliveryServicesResponse") as DeliveryServicesResponseMock:
			DeliveryServicesResponseMock.model_validate_json = Mock()
			DeliveryServicesResponseMock.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert await api.get_delivery_services() == "DESERIALIZED DATA"
				DeliveryServicesResponseMock.model_validate_json.assert_called_once_with("RAW DATA")
				validate_response_mock.assert_called_once_with(session.response)
				assert session.last_call_method == "GET"
				assert session.last_call_url == api.router.delivery_services()
				assert session.last_call_params is None



from ya_market_api.sync_api import SyncAPI
from ya_market_api.generic.requests.auth import APIKeyAuth
from ya_market_api.const import Header
from ya_market_api.guide.sync_api import SyncGuideAPI
from ya_market_api.feedback.sync_api import SyncFeedbackAPI
from ya_market_api.base.sync_config import SyncConfig

from unittest.mock import patch

from requests.sessions import Session


class TestSyncAPI:
	def test___init__(self):
		session = Session()
		config = SyncConfig(session, "")
		api = SyncAPI(config)
		assert api.config.session is session
		assert isinstance(api.guide, SyncGuideAPI)
		assert isinstance(api.feedback, SyncFeedbackAPI)
		assert api.feedback.config.business_id is None

		api.config.business_id = 0
		assert api.feedback.business_id == 0

	def test_build(self):
		with patch.object(SyncAPI, "make_session") as make_session_mock:
			make_session_mock.return_value = "SESSION"
			api = SyncAPI.build("API_KEY")
			assert isinstance(api, SyncAPI)
			assert api.config.session == "SESSION"
			assert api.config.business_id is None
			make_session_mock.assert_called_once_with("API_KEY")

			api = SyncAPI.build("API_KEY", business_id=0)
			assert api.config.business_id == 0

			api = SyncAPI.build("API_KEY", business_id=0, base_url="http://localhost")
			assert api.config.business_id == 0
			assert api.config.base_url == "http://localhost"

	def test_make_session(self):
		session = SyncAPI.make_session("API_KEY")
		assert isinstance(session, Session)
		assert isinstance(session.auth, APIKeyAuth)
		assert session.auth.api_key == "API_KEY"
		assert session.auth.header_label == Header.API_KEY.value



from ya_market_api.const import Header, BASE_URL
from ya_market_api.generic.requests.auth import APIKeyAuth
from ya_market_api.guide.sync_api import SyncGuideAPI
from ya_market_api.feedback.sync_api import SyncFeedbackAPI
from ya_market_api.offer.sync_api import SyncOfferAPI
from ya_market_api.campaign.sync_api import SyncCampaignAPI
from ya_market_api.order.sync_api import SyncOrderAPI
from ya_market_api.base.sync_config import SyncConfig

from typing import Optional

from requests.sessions import Session


class SyncAPI:
	guide: SyncGuideAPI
	feedback: SyncFeedbackAPI
	offer: SyncOfferAPI
	campaign: SyncCampaignAPI
	order: SyncOrderAPI
	config: SyncConfig

	def __init__(self, config: SyncConfig) -> None:
		self.config = config
		self.guide = SyncGuideAPI(config)
		self.feedback = SyncFeedbackAPI(config)
		self.offer = SyncOfferAPI(config)
		self.campaign = SyncCampaignAPI(config)
		self.order = SyncOrderAPI(config)

	@classmethod
	def build(
		cls,
		api_key: str,
		*,
		base_url: str = BASE_URL,
		business_id: Optional[int] = None,
		campaign_id: Optional[int] = None,
	) -> "SyncAPI":
		config = SyncConfig(
			cls.make_session(api_key),
			base_url,
			business_id=business_id,
			campaign_id=campaign_id,
		)

		return cls(config)

	@staticmethod
	def make_session(api_key: str) -> Session:
		session = Session()
		session.auth = APIKeyAuth(api_key, Header.API_KEY.value)

		return session



from ya_market_api.base.dataclass import FlippingPager
from ya_market_api.campaign.const import APIAvailabilityStatus, PlacementType

from typing import Optional, List

from pydantic.main import BaseModel
from pydantic.fields import Field
from pydantic.functional_validators import field_validator


class Request(BaseModel):
	page: Optional[int] = None
	page_size: Optional[int] = Field(default=None, serialization_alias="pageSize")

	@field_validator("page", mode="after")
	@classmethod
	def page_is_valid(cls, value: Optional[int]) -> Optional[int]:
		if value is None:
			return None

		if value < 1 or value > 10_000:
			raise ValueError("The page cannot be less than 1 or greater than 10000")

		return value


class Business(BaseModel):
	id: Optional[int] = None
	name: Optional[str] = None


class Campaign(BaseModel):
	api_availability: Optional[APIAvailabilityStatus] = Field(default=None, validation_alias="apiAvailability")
	business: Optional[Business] = None
	client_id: Optional[int] = Field(default=None, deprecated=True, validation_alias="clientId")
	domain: Optional[str] = None
	id: Optional[int] = None
	placement_type: Optional[PlacementType] = Field(default=None, validation_alias="placementType")


class Response(BaseModel):
	campaigns: List[Campaign]
	pager: Optional[FlippingPager] = None



from ya_market_api.feedback.dataclass.list import Request, Feedback
from ya_market_api.feedback.const import ReactionStatus

import pytest
from arrow import get


class TestRequest:
	def test___init__(self):
		request = Request(feedback_ids=(1, 2, 3), limit=25, page_token="page-token")
		assert request.feedback_ids == (1, 2, 3)
		assert request.limit == 25
		assert request.page_token == "page-token"

		request = Request(datetime_from=get(2025, 1, 1), datetime_to=get(2025, 2, 1))
		assert request.datetime_from == get(2025, 1, 1)
		assert request.datetime_to == get(2025, 2, 1)

	def test_limit_is_valid(self):
		assert Request.limit_is_valid(None) is None
		assert Request.limit_is_valid(25) == 25

		with pytest.raises(ValueError, match="The limit cannot be less than 1 or greater than 50"):
			Request.limit_is_valid(0)

		with pytest.raises(ValueError, match="The limit cannot be less than 1 or greater than 50"):
			Request.limit_is_valid(51)

	def test_rating_values_are_valid(self):
		assert Request.rating_values_are_valid(None) is None
		assert Request.rating_values_are_valid([]) == []
		assert Request.rating_values_are_valid([1, 2, 3]) == [1, 2, 3]

		with pytest.raises(ValueError, match=r"0 is not valid rating value. Must be in range \[1\.\.5\]"):
			assert Request.rating_values_are_valid([3, 2, 1, 0])

		with pytest.raises(ValueError, match=r"6 is not valid rating value. Must be in range \[1\.\.5\]"):
			assert Request.rating_values_are_valid([4, 5, 6])

	def test_serialize_optional_arrow(self):
		request = Request(feedback_ids=[1])

		assert request.serialize_optional_arrow(None) is None
		assert request.serialize_optional_arrow(get(2025, 1, 1, 12, 30, 30)) == "2025-01-01T12:30:30+00:00"

	def test_serialize_reaction_status(self):
		request = Request(feedback_ids=[1])

		assert request.serialize_reaction_status(None) is None
		assert request.serialize_reaction_status(ReactionStatus.ALL) == ReactionStatus.ALL.value

	def test_model_dump_request_params(self):
		request = Request(feedback_ids=[1, 2, 3], limit=50, page_token="page-token")
		assert request.model_dump_request_params() == {"limit": 50, "page_token": "page-token"}

	def test_model_dump_request_payload(self):
		request = Request(feedback_ids=[1, 2, 3], limit=50, page_token="page-token")
		assert request.model_dump_request_payload() == {"feedbackIds": [1, 2, 3]}

		request = Request(datetime_from=get(2025, 1, 1, 12, 30, 30), datetime_to=get(2025, 2, 1))
		assert request.model_dump_request_payload() == {
			"dateTimeFrom": "2025-01-01T12:30:30+00:00",
			"dateTimeTo": "2025-02-01T00:00:00+00:00",
		}

		request.feedback_ids = [1, 2, 3]

		with pytest.warns(UserWarning, match="When using feedback_id, the other parameters will be ignored"):
			assert request.model_dump_request_payload() == {"feedbackIds": [1, 2, 3]}


class TestFeedback:
	def test_validate_arrow(self):
		assert Feedback.validate_arrow("2025-01-01T12:30:30.500+03:00") == get(2025, 1, 1, 12, 30, 30, 500000, tzinfo="Europe/Moscow")

		with pytest.raises(ValueError, match="Raw value must be of str type"):
			Feedback.validate_arrow(5)



from ya_market_api.guide.dataclass.delivery_services import Response as DeliveryServicesResponse
from ya_market_api.guide.dataclass.token_info import Response as TokenInfoResponse


__all__ = ["DeliveryServicesResponse", "TokenInfoResponse"]



from ya_market_api.base.const import CurrencyType
from ya_market_api.base.dataclass import Region, GPS
from ya_market_api.base.convert import (
	optional_str_time_to_optional_time, optional_str_date_to_optional_arrow, str_date_to_arrow,
	optional_str_datetime_to_optional_arrow, str_datetime_to_arrow,
)
from ya_market_api.order.const import (
	PaymentMethod, PaymentType, OrderStatus, OrderSubstatus, OrderTaxSystem, OrderBuyerType, OrderSubsidyType,
	OrderVatType, OrderItemTag, OrderItemInstanceType, OrderDeliveryPartnerType, OrderDeliveryType,
	OrderDeliveryDispatchType, OrderDeliveryEacType, OrderLiftType, OrderItemStatusType, OrderPromoType,
	OrderItemSubsidyType,
)

from decimal import Decimal
from typing import List, Optional
from datetime import time

from pydantic.main import BaseModel
from pydantic.fields import Field
from pydantic.config import ConfigDict
from pydantic.functional_validators import field_validator
from arrow import Arrow


class Request(BaseModel):
	order_id: int


class OrderBuyer(BaseModel):
	id: Optional[str] = None
	type: OrderBuyerType
	first_name: Optional[str] = Field(default=None, validation_alias="firstName")
	last_name: Optional[str] = Field(default=None, validation_alias="lastName")
	middle_name: Optional[str] = Field(default=None, validation_alias="middleName")


class OrderDeliveryDates(BaseModel):
	model_config = ConfigDict(arbitrary_types_allowed=True)

	from_date: Arrow = Field(validation_alias="fromDate")
	from_time: Optional[time] = Field(default=None, validation_alias="fromTime")
	real_delivery_date: Optional[Arrow] = Field(default=None, validation_alias="realDeliveryDate")
	to_date: Optional[Arrow] = Field(default=None, validation_alias="toDate")
	to_time: Optional[time] = Field(default=None, validation_alias="toTime")

	@field_validator("from_date", mode="before")
	@classmethod
	def dates_must_be_arrow(cls, value: str) -> Arrow:
		return str_date_to_arrow(value)

	@field_validator("from_time", "to_time", mode="before")
	@classmethod
	def optional_times_must_be_optional_time(cls, value: Optional[str]) -> Optional[time]:
		return optional_str_time_to_optional_time(value)

	@field_validator("real_delivery_date", "to_date", mode="before")
	@classmethod
	def optional_dates_must_be_optional_arrow(cls, value: Optional[str]) -> Optional[Arrow]:
		return optional_str_date_to_optional_arrow(value)


class OrderDeliveryAddress(BaseModel):
	apartment: Optional[str] = None
	block: Optional[str] = None
	building: Optional[str] = None
	city: Optional[str] = None
	country: Optional[str] = None
	district: Optional[str] = None
	entrance: Optional[str] = None
	entryphone: Optional[str] = None
	estate: Optional[str] = None
	floor: Optional[str] = None
	gps: Optional[GPS] = None
	house: Optional[str] = None
	phone: Optional[str] = None
	postcode: Optional[str] = None
	recipient: Optional[str] = None
	street: Optional[str] = None
	subway: Optional[str] = None


class OrderCourier(BaseModel):
	full_name: Optional[str] = Field(default=None, validation_alias="fullName")
	phone: Optional[str] = None
	phone_extension: Optional[str] = Field(default=None, validation_alias="phoneExtension")
	vehicle_description: Optional[str] = Field(default=None, validation_alias="vehicleDescription")
	vehicle_number: Optional[str] = Field(default=None, validation_alias="vehicleNumber")


class OrderTrack(BaseModel):
	delivery_service_id: int = Field(validation_alias="deliveryServiceId")
	track_code: Optional[str] = Field(default=None, validation_alias="trackCode")


class OrderParcelBox(BaseModel):
	id: int
	fulfilment_id: str = Field(validation_alias="fulfilmentId")


class OrderShipment(BaseModel):
	model_config = ConfigDict(arbitrary_types_allowed=True)

	id: Optional[int] = None
	boxes: List[OrderParcelBox] = Field(default_factory=list)
	shipment_date: Optional[Arrow] = Field(default=None, validation_alias="shipmentDate")
	shipment_time: Optional[time] = Field(default=None, validation_alias="shipmentTime")
	tracks: List[OrderTrack] = Field(default_factory=list)

	@field_validator("shipment_date", mode="before")
	@classmethod
	def optional_dates_must_be_optional_arrow(cls, value: Optional[str]) -> Optional[Arrow]:
		return optional_str_date_to_optional_arrow(value)

	@field_validator("shipment_time", mode="before")
	@classmethod
	def optional_times_must_be_optional_time(cls, value: Optional[str]) -> Optional[time]:
		return optional_str_time_to_optional_time(value)


class OrderDelivery(BaseModel):
	model_config = ConfigDict(arbitrary_types_allowed=True)

	dates: OrderDeliveryDates
	delivery_partner_type: OrderDeliveryPartnerType = Field(validation_alias="deliveryPartnerType")
	delivery_service_id: int = Field(validation_alias="deliveryServiceId")
	service_name: str = Field(validation_alias="serviceName")
	type: OrderDeliveryType
	address: Optional[OrderDeliveryAddress] = None
	courier: Optional[OrderCourier] = None
	dispatch_type: Optional[OrderDeliveryDispatchType] = Field(default=None, validation_alias="dispatchType")
	eac_code: Optional[str] = Field(default=None, validation_alias="eacCode")
	eac_type: Optional[OrderDeliveryEacType] = Field(default=None, validation_alias="eacType")
	estimated: Optional[bool] = None
	id: Optional[str] = Field(default=None, deprecated=True)
	lift_price: Optional[Decimal] = Field(default=None, decimal_places=2, validation_alias="liftPrice")
	lift_type: Optional[OrderLiftType] = Field(default=None, validation_alias="liftType")
	outlet_code: Optional[str] = Field(default=None, validation_alias="outletCode")
	outlet_storage_limit_date: Optional[Arrow] = Field(default=None, validation_alias="outletStorageLimitDate")
	price: Optional[Decimal] = Field(default=None, deprecated=True, decimal_places=2)
	region: Optional[Region] = None
	shipments: List[OrderShipment] = Field(default_factory=list)
	tracks: List[OrderTrack] = Field(default_factory=list)
	vat: Optional[OrderVatType] = None

	@field_validator("outlet_storage_limit_date", mode="before")
	@classmethod
	def optional_dates_must_be_optional_arrow(cls, value: Optional[str]) -> Optional[Arrow]:
		return optional_str_date_to_optional_arrow(value)


class OrderItemDetail(BaseModel):
	model_config = ConfigDict(arbitrary_types_allowed=True)

	item_count: int = Field(validation_alias="itemCount")
	item_status: OrderItemStatusType = Field(validation_alias="itemStatus")
	updated_at: Arrow = Field(validation_alias="updateDate")

	@field_validator("updated_at", mode="before")
	@classmethod
	def dates_must_be_arrow(cls, value: str) -> Arrow:
		return str_date_to_arrow(value)


class OrderItemInstance(BaseModel):
	cis: Optional[str] = None
	cis_full: Optional[str] = Field(default=None, validation_alias="cisFull")
	country_code: Optional[str] = Field(default=None, validation_alias="countryCode")
	gtd: Optional[str] = None
	rnpt: Optional[str] = None
	uin: Optional[str] = None


class OrderItemPromo(BaseModel):
	subsidy: Decimal = Field(decimal_places=2)
	type: OrderPromoType
	discount: Optional[Decimal] = Field(default=None, decimal_places=2)
	market_promo_id: Optional[str] = Field(default=None, validation_alias="marketPromoId")
	shop_promo_id: Optional[str] = Field(default=None, validation_alias="shopPromoId")


class OrderItemSubsidy(BaseModel):
	amount: Decimal = Field(decimal_places=2)
	type: OrderItemSubsidyType


class OrderItem(BaseModel):
	buyer_price: Decimal = Field(decimal_places=2, validation_alias="buyerPrice")
	buyer_price_before_discount: Decimal = Field(decimal_places=2, validation_alias="buyerPriceBeforeDiscount")
	count: int
	id: int
	offer_id: str = Field(validation_alias="offerId")
	offer_name: str = Field(validation_alias="offerName")
	price: Decimal = Field(decimal_places=2)
	vat: OrderVatType
	details: List[OrderItemDetail] = Field(default_factory=list, deprecated=True)
	instances: List[OrderItemInstance] = Field(default_factory=list)
	partner_warehouse_id: Optional[str] = Field(default=None, deprecated=True, validation_alias="partnerWarehouseId")
	price_before_discount: Optional[Decimal] = Field(
		default=None, decimal_places=2, deprecated=True, validation_alias="priceBeforeDiscount",
	)
	promos: List[OrderItemPromo] = Field(default_factory=list)
	required_instance_types: List[OrderItemInstanceType] = Field(
		default_factory=list, validation_alias="requiredInstanceTypes",
	)
	shop_sku: Optional[str] = Field(default=None, deprecated=True, validation_alias="shopSku")
	subsidies: List[OrderItemSubsidy] = Field(default_factory=list)
	subsidy: Optional[Decimal] = Field(default=None, deprecated=True, decimal_places=2)
	tags: List[OrderItemTag] = Field(default_factory=list)


class OrderSubsidy(BaseModel):
	amount: Decimal = Field(decimal_places=2)
	type: OrderSubsidyType


class Order(BaseModel):
	model_config = ConfigDict(arbitrary_types_allowed=True)

	id: int
	buyer: OrderBuyer
	buyer_items_total_before_discount: Decimal = Field(
		decimal_places=2,
		validation_alias="buyerItemsTotalBeforeDiscount",
	)
	created_at: Arrow = Field(validation_alias="creationDate")
	currency: CurrencyType
	delivery: OrderDelivery
	delivery_total: Decimal = Field(decimal_places=2, validation_alias="deliveryTotal")
	fake: bool
	items: List[OrderItem]
	items_total: Decimal = Field(decimal_places=2, validation_alias="itemsTotal")
	payment_method: PaymentMethod = Field(validation_alias="paymentMethod")
	payment_type: PaymentType = Field(validation_alias="paymentType")
	status: OrderStatus
	substatus: OrderSubstatus
	tax_system: OrderTaxSystem = Field(validation_alias="taxSystem")
	buyer_items_total: Optional[Decimal] = Field(
		default=None, decimal_places=2, deprecated=True, validation_alias="buyerItemsTotal",
	)
	buyer_total: Optional[Decimal] = Field(
		default=None, decimal_places=2, deprecated=True, validation_alias="buyerTotal",
	)
	buyer_total_before_discount: Optional[Decimal] = Field(
		default=None,
		decimal_places=2,
		deprecated=True,
		validation_alias="buyerTotalBeforeDiscount",
	)
	cancel_requested: Optional[bool] = Field(default=None, validation_alias="cancelRequested")
	# Format in docs: YYYY-MM-DD, but example contains time, so we use only date
	expired_at: Optional[Arrow] = Field(default=None, validation_alias="expiryDate")
	external_order_id: Optional[str] = Field(default=None, validation_alias="externalOrderId")
	notes: Optional[str] = None
	subsidies: List[OrderSubsidy] = Field(default_factory=list)
	updated_at: Optional[Arrow] = Field(default=None, validation_alias="updatedAt")

	@field_validator("created_at", mode="before")
	@classmethod
	def datetimes_must_be_arrow(cls, value: str) -> Arrow:
		return str_datetime_to_arrow(value)

	@field_validator("expired_at", mode="before")
	@classmethod
	def optional_dates_must_be_optional_arrow(cls, value: Optional[str]) -> Optional[Arrow]:
		return optional_str_date_to_optional_arrow(value)

	@field_validator("updated_at", mode="before")
	@classmethod
	def optional_datetimes_must_be_optional_arrow(cls, value: Optional[str]) -> Optional[Arrow]:
		return optional_str_datetime_to_optional_arrow(value)


class Response(BaseModel):
	order: Optional[Order] = None



from ya_market_api.generic.requests.auth import APIKeyAuth

from requests.models import Request


class TestAPIKeyAuth:
	def test___call__(self):
		auth = APIKeyAuth("API key")
		request = Request()
		request = auth(request)
		assert request.headers == {"x-api-key": "API key"}

		auth.header_label = "Authorization"
		request = Request()
		request = auth(request)
		assert request.headers == {"Authorization": "API key"}



from typing import List

from pydantic.main import BaseModel
from pydantic.fields import Field


class DeliveryService(BaseModel):
	id: int
	name: str


class Result(BaseModel):
	delivery_services: List[DeliveryService] = Field(default_factory=list, validation_alias="deliveryService")


class Response(BaseModel):
	result: Result = Field(default_factory=Result)



from ya_market_api.base.dataclass.region import Region

from typing import List, Optional

from pydantic.main import BaseModel
from pydantic.fields import Field


class Request(BaseModel):
	region_id: int


class ResponsePaging(BaseModel):
	next_page_token: Optional[str] = Field(None, validation_alias="nextPageToken")


class Response(BaseModel):
	regions: List[Region]
	paging: Optional[ResponsePaging] = None



from ya_market_api.feedback.dataclass.comment_add import Request, RequestComment


class TestRequest:
	def test_create(self):
		request = Request.create(512, "comment-text")
		assert isinstance(request, Request)
		assert isinstance(request.comment, RequestComment)
		assert request.feedback_id == 512
		assert request.comment.text == "comment-text"
		assert request.comment.parent_id is None

		request = Request.create(512, "comment-text", parent_id=1024)
		assert request.feedback_id == 512
		assert request.comment.text == "comment-text"
		assert request.comment.parent_id == 1024



from tests.fake_async_session import FakeAsyncSession
from ya_market_api.offer.async_api import AsyncOfferAPI
from ya_market_api.offer.dataclass import OfferListByBusinessRequest
from ya_market_api.base.async_config import AsyncConfig

from unittest.mock import patch, Mock

import pytest


class TestAsyncOfferAPI:
	@pytest.mark.asyncio()
	async def test_get_offer_list_by_business(self):
		session = FakeAsyncSession("RAW DATA")
		config = AsyncConfig(session, "", business_id=1)		# type: ignore - for testing purposes
		api = AsyncOfferAPI(config)
		request = OfferListByBusinessRequest(limit=10, page_token="page-token", category_ids=(1, 2, 3))

		with patch("ya_market_api.offer.async_api.OfferListByBusinessResponse") as OfferListByBusinessResponseMock:
			OfferListByBusinessResponseMock.model_validate_json = Mock()
			OfferListByBusinessResponseMock.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert await api.get_offer_list_by_business() == "DESERIALIZED DATA"
				OfferListByBusinessResponseMock.model_validate_json.assert_called_once_with("RAW DATA")
				validate_response_mock.assert_called_once_with(session.response)
				assert session.last_call_method == "POST"
				assert session.last_call_url == api.router.offer_list_by_business(1)
				assert session.last_call_json == {}
				assert session.last_call_params == {}

				assert await api.get_offer_list_by_business(request) == "DESERIALIZED DATA"
				assert session.last_call_method == "POST"
				assert session.last_call_url == api.router.offer_list_by_business(1)
				assert session.last_call_json == {"categoryIds": (1, 2, 3)}
				assert session.last_call_params == {"limit": 10, "page_token": "page-token"}



from ya_market_api.guide.region.sync_api import SyncGuideRegionAPI
from ya_market_api.guide.region.dataclass import RegionSearchRequest, RegionInfoRequest, RegionChildrenRequest
from ya_market_api.base.sync_config import SyncConfig

from unittest.mock import patch, Mock


class TestSyncGuideRegionAPI:
	def test_get_region_countries(self):
		session = Mock()
		session.post = Mock()
		session.post.return_value = Mock()
		session.post.return_value.text = "RAW DATA"
		config = SyncConfig(session, "", business_id=1)
		api = SyncGuideRegionAPI(config)

		with patch("ya_market_api.guide.region.sync_api.RegionCountriesResponse") as RegionCountriesResponseMock:
			RegionCountriesResponseMock.model_validate_json = Mock()
			RegionCountriesResponseMock.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert api.get_region_countries() == "DESERIALIZED DATA"
				RegionCountriesResponseMock.model_validate_json.assert_called_once_with("RAW DATA")
				validate_response_mock.assert_called_once_with(session.post.return_value)
				session.post.assert_called_once_with(url=api.router.region_countries(), json="")

	def test_search_region(self):
		session = Mock()
		session.get = Mock()
		session.get.return_value = Mock()
		session.get.return_value.text = "RAW DATA"
		config = SyncConfig(session, "", business_id=1)
		api = SyncGuideRegionAPI(config)
		request = RegionSearchRequest(name="REGION_NAME", limit=100, page_token="PAGE_TOKEN")

		with patch("ya_market_api.guide.region.sync_api.RegionSearchResponse") as RegionSearchResponseMock:
			RegionSearchResponseMock.model_validate_json = Mock()
			RegionSearchResponseMock.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert api.search_region(request) == "DESERIALIZED DATA"
				RegionSearchResponseMock.model_validate_json.assert_called_once_with("RAW DATA")
				validate_response_mock.assert_called_once_with(session.get.return_value)
				session.get.assert_called_once_with(
					url=api.router.region_search(),
					params={"name": "REGION_NAME", "limit": 100, "page_token": "PAGE_TOKEN"},
				)

	def test_get_region_info(self):
		session = Mock()
		session.get = Mock()
		session.get.return_value = Mock()
		session.get.return_value.text = "RAW DATA"
		config = SyncConfig(session, "", business_id=1)
		api = SyncGuideRegionAPI(config)
		request = RegionInfoRequest(region_id=512)

		with patch("ya_market_api.guide.region.sync_api.RegionInfoResponse") as RegionInfoResponseMock:
			RegionInfoResponseMock.model_validate_json = Mock()
			RegionInfoResponseMock.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert api.get_region_info(request) == "DESERIALIZED DATA"
				RegionInfoResponseMock.model_validate_json.assert_called_once_with("RAW DATA")
				validate_response_mock.assert_called_once_with(session.get.return_value)
				session.get.assert_called_once_with(url=api.router.region_info(512))

	def test_get_region_children(self):
		session = Mock()
		session.get = Mock()
		session.get.return_value = Mock()
		session.get.return_value.text = "RAW DATA"
		config = SyncConfig(session, "", business_id=1)
		api = SyncGuideRegionAPI(config)
		request = RegionChildrenRequest(region_id=512, page=1, page_size=100)

		with patch("ya_market_api.guide.region.sync_api.RegionChildrenResponse") as RegionChildrenResponseMock:
			RegionChildrenResponseMock.model_validate_json = Mock()
			RegionChildrenResponseMock.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert api.get_region_children(request) == "DESERIALIZED DATA"
				RegionChildrenResponseMock.model_validate_json.assert_called_once_with("RAW DATA")
				validate_response_mock.assert_called_once_with(session.get.return_value)
				session.get.assert_called_once_with(
					url=api.router.region_children(512),
					params={"page": 1, "pageSize": 100},
				)



from ya_market_api.base.dataclass import BaseResponse
from ya_market_api.base.const import CurrencyType
from ya_market_api.offer.const import (
	CatalogLanguageType, OfferCardStatusType, AgeUnit, CampaignStatusType, CommodityCodeType,
	OfferConditionQualityType, OfferConditionType, TimeUnit, MediaFileUploadState, SellingProgramType,
	SellingProgramStatusType, OfferType, ShowcaseType,
)

from typing import FrozenSet, Optional, Collection, List, Dict, Any, ClassVar, overload
from warnings import warn

from pydantic.main import BaseModel
from pydantic.fields import Field
from pydantic.config import ConfigDict
from pydantic.functional_validators import field_validator
from pydantic.functional_serializers import field_serializer
from arrow import Arrow, get as get_arrow


CardStatusesParam = Optional[Collection[OfferCardStatusType]]
CategoryIdsParam = Optional[Collection[int]]
OfferIdsParam = Optional[Collection[str]]
TagsParam = Optional[Collection[str]]
VendorNamesParam = Optional[Collection[str]]


class Request(BaseModel):
	QUERY_PARAMS: ClassVar[FrozenSet[str]] = frozenset({"language", "limit", "page_token"})
	S11N_ALIAS_OFFER_IDS: ClassVar[str] = "offerIds"
	model_config = ConfigDict(arbitrary_types_allowed=True)

	# query params
	language: Optional[CatalogLanguageType] = None
	limit: Optional[int] = Field(default=None)
	page_token: Optional[str] = None

	# payload v1
	archived: Optional[bool] = None
	card_statuses: CardStatusesParam = Field(default=None, serialization_alias="cardStatuses")
	category_ids: CategoryIdsParam = Field(default=None, serialization_alias="categoryIds")
	tags: TagsParam = None
	vendor_names: VendorNamesParam = Field(default=None, serialization_alias="vendorNames")

	# payload v2
	offer_ids: OfferIdsParam = Field(default=None, serialization_alias=S11N_ALIAS_OFFER_IDS)

	@overload
	def __init__(self, *, offer_ids: Collection[str]) -> None: ...
	@overload
	def __init__(
		self,
		*,
		language: Optional[CatalogLanguageType] = None,
		limit: Optional[int] = None,
		page_token: Optional[str] = None,
		archived: Optional[bool] = None,
		card_statuses: CardStatusesParam = None,
		category_ids: CategoryIdsParam = None,
		tags: TagsParam = None,
		vendor_names: VendorNamesParam = None,
	) -> None: ...
	def __init__(
		self,
		*,
		language: Optional[CatalogLanguageType] = None,
		limit: Optional[int] = None,
		page_token: Optional[str] = None,
		archived: Optional[bool] = None,
		card_statuses: CardStatusesParam = None,
		category_ids: CategoryIdsParam = None,
		tags: TagsParam = None,
		vendor_names: VendorNamesParam = None,
		offer_ids: OfferIdsParam = None,
	) -> None:
		super().__init__(
			language=language,
			limit=limit,
			page_token=page_token,
			archived=archived,
			card_statuses=card_statuses,
			category_ids=category_ids,
			tags=tags,
			vendor_names=vendor_names,
			offer_ids=offer_ids,
		)

	@field_validator("limit", mode="after")
	@classmethod
	def limit_must_be_greater_than_0(cls, value: Optional[int]) -> Optional[int]:
		if value is None:
			return None

		if value < 1:
			raise ValueError("Limit cannot be less than 1")

		return value

	@field_validator("card_statuses", mode="after")
	@classmethod
	def card_statuses_must_be_filled(cls, value: CardStatusesParam) -> CardStatusesParam:
		if value is None:
			return None

		if len(value) < 1:
			raise ValueError("Card statuses length cannot be less than 1")

		return value

	@field_validator("category_ids", mode="after")
	@classmethod
	def category_ids_must_be_filled(cls, value: CategoryIdsParam) -> CategoryIdsParam:
		if value is None:
			return None

		if len(value) < 1:
			raise ValueError("Category ids length cannot be less than 1")

		return value

	@field_validator("offer_ids", mode="after")
	@classmethod
	def offer_ids_must_be_filled(cls, value: OfferIdsParam) -> OfferIdsParam:
		if value is None:
			return None

		if len(value) < 1:
			raise ValueError("Offer ids length cannot be less than 1")
		elif len(value) > 200:
			raise ValueError("Offer ids length cannot be greater than 200")

		for offer_id in value:
			if len(offer_id) < 1:
				raise ValueError("Offer id length cannot be less than 1")
			elif len(offer_id) > 250:
				raise ValueError("Offer id length cannot be greater than 250")

		return value

	@field_validator("tags", mode="after")
	@classmethod
	def tags_must_be_filled(cls, value: TagsParam) -> TagsParam:
		if value is None:
			return None

		if len(value) < 1:
			raise ValueError("Tags length cannot be less than 1")

		return value

	@field_validator("vendor_names", mode="after")
	@classmethod
	def vendor_names_must_be_filled(cls, value: VendorNamesParam) -> VendorNamesParam:
		if value is None:
			return None

		if len(value) < 1:
			raise ValueError("Vendor names length cannot be less than 1")

		return value

	@field_serializer("language")
	def serialize_language(self, value: Optional[CatalogLanguageType]) -> Optional[str]:
		if value is None:
			return None

		return value.value

	@field_serializer("card_statuses")
	def serialize_card_statuses(self, value: CardStatusesParam) -> Optional[List[str]]:
		if value is None:
			return None

		return [i.value for i in value]

	def model_dump_request_params(self) -> Dict[str, Any]:
		result = self.model_dump(include=self.QUERY_PARAMS, by_alias=True, exclude_none=True)

		if self.offer_ids is not None and len(result) != 0:
			warn("When using offer_ids, the other query parameters will be ignored")
			result = {}

		return result

	def model_dump_request_payload(self) -> Dict[str, Any]:
		result = self.model_dump(exclude=self.QUERY_PARAMS, by_alias=True, exclude_none=True)

		if self.offer_ids is not None and len(result) != 1:
			warn("When using offer_ids, the other parameters will be ignored")
			result = {self.S11N_ALIAS_OFFER_IDS: result[self.S11N_ALIAS_OFFER_IDS]}

		return result


############
# RESPONSE #
############
class Mapping(BaseModel):
	market_category_id: Optional[int] = Field(default=None, validation_alias="marketCategoryId")
	market_category_name: Optional[str] = Field(default=None, validation_alias="marketCategoryName")
	market_model_id: Optional[int] = Field(default=None, deprecated=True, validation_alias="marketModelId")
	market_model_name: Optional[str] = Field(default=None, deprecated=True, validation_alias="marketModelName")
	market_sku: Optional[int] = Field(default=None, validation_alias="marketSku")
	market_sku_name: Optional[str] = Field(default=None, validation_alias="marketSkuName")


class Price(BaseModel):
	model_config = ConfigDict(arbitrary_types_allowed=True)

	currency_id: CurrencyType = Field(validation_alias="currencyId")
	updated_at: Arrow = Field(validation_alias="updatedAt")
	value: float

	@field_validator("updated_at", mode="before")
	@classmethod
	def datetimes_must_be_arrow(cls, value: str) -> Arrow:
		return get_arrow(value)


class Age(BaseModel):
	unit: AgeUnit = Field(validation_alias="ageUnit")
	value: float


class PriceWithDiscount(BaseModel):
	model_config = ConfigDict(arbitrary_types_allowed=True)

	updated_at: Arrow = Field(validation_alias="updatedAt")
	currency_id: Optional[CurrencyType] = Field(default=None, validation_alias="currencyId")
	discount_base: Optional[int] = Field(default=None, validation_alias="discountBase")
	value: Optional[float] = None

	@field_validator("updated_at", mode="before")
	@classmethod
	def datetimes_must_be_arrow(cls, value: str) -> Arrow:
		return get_arrow(value)


class CampaignStatus(BaseModel):
	campaign_id: int = Field(validation_alias="campaignId")
	status: CampaignStatusType


class CommodityCode(BaseModel):
	code: str
	type: CommodityCodeType


class OfferCondition(BaseModel):
	quality: Optional[OfferConditionQualityType] = None
	reason: Optional[str] = None
	type: Optional[OfferConditionType] = None


class TimePeriod(BaseModel):
	period: int = Field(validation_alias="timePeriod")
	unit: TimeUnit = Field(validation_alias="timeUnit")
	comment: Optional[str] = None


class OfferManual(BaseModel):
	url: str
	title: Optional[str] = None


class OfferMediaFile(BaseModel):
	title: Optional[str] = None
	upload_state: Optional[MediaFileUploadState] = Field(default=None, validation_alias="uploadState")
	url: Optional[str] = None


class OfferMediaFiles(BaseModel):
	first_video_as_cover: Optional[bool] = Field(default=None, validation_alias="firstVideoAsCover", deprecated=True)
	manuals: List[OfferMediaFile] = Field(default_factory=list)
	pictures: List[OfferMediaFile] = Field(default_factory=list)
	videos: List[OfferMediaFile] = Field(default_factory=list)


class OfferSellingProgram(BaseModel):
	type: SellingProgramType = Field(validation_alias="sellingProgram")
	status: SellingProgramStatusType


class OfferWeightDimensions(BaseModel):
	height: float
	length: float
	weight: float
	width: float


class OfferParam(BaseModel):
	name: str
	value: str


class Offer(BaseModel):
	id: str = Field(validation_alias="offerId")
	additional_expenses: Optional[Price] = Field(default=None, validation_alias="additionalExpenses")
	adult: Optional[bool] = None
	age: Optional[Age] = None
	archived: Optional[bool] = None
	barcodes: List[str] = Field(default_factory=list)
	basic_price: Optional[PriceWithDiscount] = Field(default=None, validation_alias="basicPrice")
	box_count: Optional[int] = Field(default=None, validation_alias="boxCount")
	campaigns: List[CampaignStatus] = Field(default_factory=list)
	card_status: Optional[OfferCardStatusType] = Field(default=None, validation_alias="cardStatus")
	category: Optional[str] = Field(default=None, deprecated=True)
	certificates: List[str] = Field(default_factory=list, validation_alias="certificates")
	commodity_codes: List[CommodityCode] = Field(default_factory=list, validation_alias="commodityCodes")
	condition: Optional[OfferCondition] = None
	customs_commodity_code: Optional[str] = Field(
		default=None, deprecated=True, validation_alias="customsCommodityCodes",
	)
	description: Optional[str] = None
	downloadable: Optional[bool] = None
	guarantee_period: Optional[TimePeriod] = Field(default=None, validation_alias="guaranteePeriod")
	life_time: Optional[TimePeriod] = Field(default=None, validation_alias="lifeTime")
	manuals: List[OfferManual] = Field(default_factory=list)
	manufacturer_countries: List[str] = Field(default_factory=list, validation_alias="manufacturerCountries")
	market_category_id: Optional[int] = Field(default=None, validation_alias="marketCategoryId")		# Seems to be abandoned
	media_files: Optional[OfferMediaFiles] = Field(default=None, validation_alias="mediaFiles")
	name: Optional[str] = None
	params: List[OfferParam] = Field(default_factory=list, deprecated=True)
	pictures: List[str] = Field(default_factory=list)
	purchase_price: Optional[Price] = Field(default=None, validation_alias="purchasePrice")
	selling_programs: List[OfferSellingProgram] = Field(default_factory=list, validation_alias="sellingParams")
	shelf_life: Optional[TimePeriod] = Field(default=None, validation_alias="shelfLife")
	tags: List[str] = Field(default_factory=list)
	type: Optional[OfferType] = None
	vendor: Optional[str] = None
	vendor_code: Optional[str] = Field(default=None, validation_alias="vendorCode")
	videos: List[str] = Field(default_factory=list)
	weight_dimensions: Optional[OfferWeightDimensions] = Field(default=None, validation_alias="weightDimensions")


class ShowcaseURL(BaseModel):
	type: ShowcaseType = Field(validation_alias="showcaseType")
	url: str = Field(validation_alias="showcaseUrl")


class OfferMapping(BaseModel):
	mapping: Optional[Mapping] = None
	offer: Optional[Offer] = None
	showcase_urls: List[ShowcaseURL] = Field(default_factory=list, validation_alias="showcaseUrls")


class Paging(BaseModel):
	next_page_token: Optional[str] = Field(default=None, validation_alias="nextPageToken")
	prev_page_token: Optional[str] = Field(default=None, validation_alias="prevPageToken")


class ResponseResult(BaseModel):
	offer_mappings: List[OfferMapping] = Field(validation_alias="offerMappings")
	paging: Optional[Paging] = None


class Response(BaseResponse):
	result: Optional[ResponseResult] = None



from ya_market_api.base.dataclass import BaseResponse
from ya_market_api.feedback.const import CommentStatus
from ya_market_api.feedback.dataclass.generic import FeedbackCommentAuthor

from typing import Optional

from pydantic.main import BaseModel
from pydantic.fields import Field


class RequestComment(BaseModel):
	parent_id: Optional[int] = Field(default=None, serialization_alias="parentId")
	text: str = Field(min_length=1)


class Request(BaseModel):
	feedback_id: int = Field(serialization_alias="feedbackId")
	comment: RequestComment

	@classmethod
	def create(cls, feedback_id: int, text: str, parent_id: Optional[int] = None) -> "Request":
		return cls(feedback_id=feedback_id, comment=RequestComment(parent_id=parent_id, text=text))


class Result(BaseModel):
	id: int
	author: FeedbackCommentAuthor
	feedback_id: int = Field(validation_alias="feedbackId")
	status: CommentStatus
	text: str
	can_modify: Optional[bool] = Field(default=None, validation_alias="canModify")
	parent_id: Optional[int] = Field(default=None, validation_alias="parentId")


class Response(BaseResponse):
	result: Optional[Result] = None



from typing import Optional

from pydantic.main import BaseModel
from pydantic.fields import Field


class FlippingPager(BaseModel):
	current_page: Optional[int] = Field(None, validation_alias="currentPage")
	from_: Optional[int] = Field(None, validation_alias="from")
	page_size: Optional[int] = Field(None, validation_alias="pageSize")
	page_count: Optional[int] = Field(None, validation_alias="pagesCount")
	to: Optional[int] = None
	total: Optional[int] = None



from ya_market_api.base.router import Router
from ya_market_api.generic.functools import cache


class CampaignRouter(Router):
	@cache
	def campaign_list(self) -> str:
		return f"{self.base_url}/campaigns"



from tests.fake_async_session import FakeAsyncSession
from ya_market_api.guide.region.async_api import AsyncGuideRegionAPI
from ya_market_api.guide.region.dataclass import RegionSearchRequest, RegionInfoRequest, RegionChildrenRequest
from ya_market_api.base.async_config import AsyncConfig

from unittest.mock import patch, Mock

import pytest


class TestAsyncGuideRegionAPI:
	@pytest.mark.asyncio()
	async def test_get_region_countries(self):
		session = FakeAsyncSession("RAW DATA")
		config = AsyncConfig(session, "", business_id=1)		# type: ignore - for testing purposes
		api = AsyncGuideRegionAPI(config)

		with patch("ya_market_api.guide.region.async_api.RegionCountriesResponse") as RegionCountriesResponseMock:
			RegionCountriesResponseMock.model_validate_json = Mock()
			RegionCountriesResponseMock.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert await api.get_region_countries() == "DESERIALIZED DATA"
				RegionCountriesResponseMock.model_validate_json.assert_called_once_with("RAW DATA")
				validate_response_mock.assert_called_once_with(session.response)
				assert session.last_call_method == "POST"
				assert session.last_call_url == api.router.region_countries()
				assert session.last_call_json == ""

	@pytest.mark.asyncio()
	async def test_get_search_region(self):
		session = FakeAsyncSession("RAW DATA")
		config = AsyncConfig(session, "", business_id=1)		# type: ignore - for testing purposes
		api = AsyncGuideRegionAPI(config)
		request = RegionSearchRequest(name="REGION_NAME", limit=100, page_token="PAGE_TOKEN")

		with patch("ya_market_api.guide.region.async_api.RegionSearchResponse") as RegionSearchResponseMock:
			RegionSearchResponseMock.model_validate_json = Mock()
			RegionSearchResponseMock.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert await api.search_region(request) == "DESERIALIZED DATA"
				RegionSearchResponseMock.model_validate_json.assert_called_once_with("RAW DATA")
				validate_response_mock.assert_called_once_with(session.response)
				assert session.last_call_method == "GET"
				assert session.last_call_url == api.router.region_search()
				assert session.last_call_params == {"name": "REGION_NAME", "limit": 100, "page_token": "PAGE_TOKEN"}

	@pytest.mark.asyncio()
	async def test_get_region_info(self):
		session = FakeAsyncSession("RAW DATA")
		config = AsyncConfig(session, "", business_id=1)		# type: ignore - for testing purposes
		api = AsyncGuideRegionAPI(config)
		request = RegionInfoRequest(region_id=512)

		with patch("ya_market_api.guide.region.async_api.RegionInfoResponse") as RegionInfoResponseMock:
			RegionInfoResponseMock.model_validate_json = Mock()
			RegionInfoResponseMock.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert await api.get_region_info(request) == "DESERIALIZED DATA"
				RegionInfoResponseMock.model_validate_json.assert_called_once_with("RAW DATA")
				validate_response_mock.assert_called_once_with(session.response)
				assert session.last_call_method == "GET"
				assert session.last_call_url == api.router.region_info(512)

	@pytest.mark.asyncio()
	async def test_get_region_children(self):
		session = FakeAsyncSession("RAW DATA")
		config = AsyncConfig(session, "", business_id=1)		# type: ignore - for testing purposes
		api = AsyncGuideRegionAPI(config)
		request = RegionChildrenRequest(region_id=512, page=1, page_size=100)

		with patch("ya_market_api.guide.region.async_api.RegionChildrenResponse") as RegionChildrenResponseMock:
			RegionChildrenResponseMock.model_validate_json = Mock()
			RegionChildrenResponseMock.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert await api.get_region_children(request) == "DESERIALIZED DATA"
				RegionChildrenResponseMock.model_validate_json.assert_called_once_with("RAW DATA")
				validate_response_mock.assert_called_once_with(session.response)
				assert session.last_call_method == "GET"
				assert session.last_call_url == api.router.region_children(512)
				assert session.last_call_params == {"page": 1, "pageSize": 100}



from ya_market_api.async_api import AsyncAPI
from ya_market_api.const import Header, BASE_URL
from ya_market_api.guide.async_api import AsyncGuideAPI
from ya_market_api.feedback.async_api import AsyncFeedbackAPI
from ya_market_api.base.async_config import AsyncConfig

from unittest.mock import patch

import pytest
from aiohttp.client import ClientSession


class TestAsyncAPI:
	@pytest.mark.asyncio()
	async def test___init__(self):
		session = ClientSession()
		config = AsyncConfig(session, "", business_id=1)
		api = AsyncAPI(config)
		assert api.config is config
		assert isinstance(api.guide, AsyncGuideAPI)
		assert isinstance(api.feedback, AsyncFeedbackAPI)
		assert api.guide.config is config
		assert api.guide.region.config is config
		assert api.feedback.config is config

		config.business_id = 0
		assert api.feedback.business_id == 0

	@pytest.mark.asyncio()
	async def test_build(self):
		with patch.object(AsyncAPI, "make_session") as make_session_mock:
			make_session_mock.return_value = "SESSION"
			api = await AsyncAPI.build("API_KEY")
			assert isinstance(api, AsyncAPI)
			assert api.config.session == "SESSION"
			assert api.config.business_id is None
			make_session_mock.assert_called_once_with("API_KEY")

			api = await AsyncAPI.build("API_KEY", business_id=0)
			assert api.config.business_id == 0
			assert api.config.base_url == BASE_URL

			api = await AsyncAPI.build("API_KEY", business_id=0, base_url="http://localhost")
			assert api.config.base_url == "http://localhost"

	@pytest.mark.asyncio()
	async def test_make_session(self):
		session = await AsyncAPI.make_session("API_KEY")
		assert isinstance(session, ClientSession)
		assert session.headers == {Header.API_KEY.value: "API_KEY"}

	@pytest.mark.asyncio()
	async def test_close(self):
		api = await AsyncAPI.build("API_KEY")
		await api.close()
		assert api.config.session.closed



from ya_market_api.base.enum_toolkit import allow_unknown

from enum import Enum


def test_allow_unknown():
	@allow_unknown
	class A(Enum):
		pass

	assert A.UNKNOWN.value == "UNKNOWN"		# type: ignore - Only for testing
	assert A("A") is A.UNKNOWN		# type: ignore - Only for testing

	@allow_unknown
	class B(Enum):
		A = "A"
		B = "B"
		UNKNOWN = "C"

	assert B.A.value == "A"
	assert B.B.value == "B"
	assert B.UNKNOWN.value == "C"
	assert B("A") is B.A
	assert B("D") is B.UNKNOWN
	assert B("F").value == "C"



from requests.auth import AuthBase
from requests.models import Request


class APIKeyAuth(AuthBase):
	def __init__(self, api_key: str, header_label: str = "x-api-key") -> None:
		self.api_key = api_key
		self.header_label = header_label

	def __call__(self, request: Request) -> Request:
		request.headers[self.header_label] = self.api_key
		return request



from ya_market_api.base.api import API
from ya_market_api.guide.router import GuideRouter


class BaseGuideAPI(API[GuideRouter]):
	@staticmethod
	def make_router(base_url: str):
		return GuideRouter(base_url)



from ya_market_api.guide.region.dataclass.children import Request

import pytest


class TestRequest:
	def test_page_is_valid(self):
		assert Request.page_is_valid(None) is None
		assert Request.page_is_valid(1) == 1
		assert Request.page_is_valid(10_000) == 10_000

		with pytest.raises(ValueError, match="The page cannot be less than 1 or greater than 10000"):
			Request.page_is_valid(0)

		with pytest.raises(ValueError, match="The page cannot be less than 1 or greater than 10000"):
			Request.page_is_valid(10_001)

	def test_page_size_is_valid(self):
		assert Request.page_size_is_valid(None) is None
		assert Request.page_size_is_valid(1) == 1
		assert Request.page_size_is_valid(10_000) == 10_000

		with pytest.raises(ValueError, match="The page_size cannot be less than 1"):
			Request.page_size_is_valid(0)



from ya_market_api.campaign.sync_api import SyncCampaignAPI
from ya_market_api.campaign.dataclass import CampaignListRequest
from ya_market_api.base.sync_config import SyncConfig

from unittest.mock import patch, Mock


class TestSyncCampaignAPI:
	def test_get_campaign_list(self):
		session = Mock()
		session.get = Mock()
		session.get.return_value = Mock()
		session.get.return_value.text = "RAW DATA"
		config = SyncConfig(session, "")
		api = SyncCampaignAPI(config)
		request = CampaignListRequest(page=10, page_size=100)

		with patch("ya_market_api.campaign.sync_api.CampaignListResponse") as CampaignListResponseMock:
			CampaignListResponseMock.model_validate_json = Mock()
			CampaignListResponseMock.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert api.get_campaign_list() == "DESERIALIZED DATA"
				CampaignListResponseMock.model_validate_json.assert_called_once_with("RAW DATA")
				validate_response_mock.assert_called_once_with(session.get.return_value)
				session.get.assert_called_once_with(url=api.router.campaign_list(), params={})

				session.get.reset_mock()
				assert api.get_campaign_list(request) == "DESERIALIZED DATA"
				session.get.assert_called_once_with(
					url=api.router.campaign_list(),
					params={"page": 10, "pageSize": 100},
				)



from ya_market_api.order.sync_api import SyncOrderAPI
from ya_market_api.order.dataclass import OrderGetRequest
from ya_market_api.base.sync_config import SyncConfig

from unittest.mock import patch, Mock


class TestSyncOrderAPI:
	def test_get_order(self):
		session = Mock()
		session.get = Mock()
		session.get.return_value = Mock()
		session.get.return_value.text = "RAW DATA"
		config = SyncConfig(session, "", campaign_id=1)
		api = SyncOrderAPI(config)
		request = OrderGetRequest(order_id=512)

		with patch("ya_market_api.order.sync_api.OrderGetResponse") as OrderGetResponseMock:
			OrderGetResponseMock.model_validate_json = Mock()
			OrderGetResponseMock.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert api.get_order(request) == "DESERIALIZED DATA"
				OrderGetResponseMock.model_validate_json.assert_called_once_with("RAW DATA")
				validate_response_mock.assert_called_once_with(session.get.return_value)
				session.get.assert_called_once_with(url=api.router.order_get(1, 512))



from ya_market_api.base.enum_toolkit import allow_unknown

from enum import Enum


@allow_unknown
class PaymentMethod(Enum):
	YANDEX = "YANDEX"		# банковской картой.
	APPLE_PAY = "APPLE_PAY"		# Apple Pay.
	GOOGLE_PAY = "GOOGLE_PAY"		# Google Pay.
	CREDIT = "CREDIT"		# в кредит.
	TINKOFF_CREDIT = "TINKOFF_CREDIT"		# в кредит в Тинькофф Банке.
	TINKOFF_INSTALLMENTS = "TINKOFF_INSTALLMENTS"		# рассрочка в Тинькофф Банке.
	EXTERNAL_CERTIFICATE = "EXTERNAL_CERTIFICATE"		# подарочным сертификатом (например, из приложения «Сбербанк Онлайн»).
	SBP = "SBP"		# через систему быстрых платежей.
	B2B_ACCOUNT_PREPAYMENT = "B2B_ACCOUNT_PREPAYMENT"		# заказ оплачивает организация.
	CARD_ON_DELIVERY = "CARD_ON_DELIVERY"		# банковской картой.
	BOUND_CARD_ON_DELIVERY = "BOUND_CARD_ON_DELIVERY"		# привязанной картой при получении.
	BNPL_BANK_ON_DELIVERY = "BNPL_BANK_ON_DELIVERY"		# супер Сплитом.
	BNPL_ON_DELIVERY = "BNPL_ON_DELIVERY"		# Сплитом.
	CASH_ON_DELIVERY = "CASH_ON_DELIVERY"		# наличными.
	B2B_ACCOUNT_POSTPAYMENT = "B2B_ACCOUNT_POSTPAYMENT"		# заказ оплачивает организация после доставки.
	UNKNOWN = "UNKNOWN"		# неизвестный тип.


@allow_unknown
class PaymentType(Enum):
	PREPAID = "PREPAID"		# оплата при оформлении заказа.
	POSTPAID = "POSTPAID"		# оплата при получении заказа.
	UNKNOWN = "UNKNOWN"		# неизвестный тип.


@allow_unknown
class OrderStatus(Enum):
	PLACING = "PLACING"		# оформляется, подготовка к резервированию.
	RESERVED = "RESERVED"		# зарезервирован, но недооформлен.
	UNPAID = "UNPAID"		# оформлен, но еще не оплачен (если выбрана оплата при оформлении).
	PROCESSING = "PROCESSING"		# находится в обработке.
	DELIVERY = "DELIVERY"		# передан в службу доставки.
	PICKUP = "PICKUP"		# доставлен в пункт самовывоза.
	DELIVERED = "DELIVERED"		# получен покупателем.
	CANCELLED = "CANCELLED"		# отменен.
	PENDING = "PENDING"		# ожидает обработки со стороны продавца.
	PARTIALLY_RETURNED = "PARTIALLY_RETURNED"		# возвращен частично.
	RETURNED = "RETURNED"		# возвращен полностью.
	UNKNOWN = "UNKNOWN"		# неизвестный статус.


@allow_unknown
class OrderSubstatus(Enum):
	RESERVATION_EXPIRED = "RESERVATION_EXPIRED"
	USER_NOT_PAID = "USER_NOT_PAID"
	USER_UNREACHABLE = "USER_UNREACHABLE"
	USER_CHANGED_MIND = "USER_CHANGED_MIND"
	USER_REFUSED_DELIVERY = "USER_REFUSED_DELIVERY"
	USER_REFUSED_PRODUCT = "USER_REFUSED_PRODUCT"
	SHOP_FAILED = "SHOP_FAILED"
	USER_REFUSED_QUALITY = "USER_REFUSED_QUALITY"
	REPLACING_ORDER = "REPLACING_ORDER"
	PROCESSING_EXPIRED = "PROCESSING_EXPIRED"
	PENDING_EXPIRED = "PENDING_EXPIRED"
	SHOP_PENDING_CANCELLED = "SHOP_PENDING_CANCELLED"
	PENDING_CANCELLED = "PENDING_CANCELLED"
	USER_FRAUD = "USER_FRAUD"
	RESERVATION_FAILED = "RESERVATION_FAILED"
	USER_PLACED_OTHER_ORDER = "USER_PLACED_OTHER_ORDER"
	USER_BOUGHT_CHEAPER = "USER_BOUGHT_CHEAPER"
	MISSING_ITEM = "MISSING_ITEM"
	BROKEN_ITEM = "BROKEN_ITEM"
	WRONG_ITEM = "WRONG_ITEM"
	PICKUP_EXPIRED = "PICKUP_EXPIRED"
	DELIVERY_PROBLEMS = "DELIVERY_PROBLEMS"
	LATE_CONTACT = "LATE_CONTACT"
	CUSTOM = "CUSTOM"
	DELIVERY_SERVICE_FAILED = "DELIVERY_SERVICE_FAILED"
	WAREHOUSE_FAILED_TO_SHIP = "WAREHOUSE_FAILED_TO_SHIP"
	DELIVERY_SERVICE_UNDELIVERED = "DELIVERY_SERVICE_UNDELIVERED"
	PREORDER = "PREORDER"
	AWAIT_CONFIRMATION = "AWAIT_CONFIRMATION"
	STARTED = "STARTED"
	PACKAGING = "PACKAGING"
	READY_TO_SHIP = "READY_TO_SHIP"
	SHIPPED = "SHIPPED"
	ASYNC_PROCESSING = "ASYNC_PROCESSING"
	WAITING_USER_INPUT = "WAITING_USER_INPUT"
	WAITING_BANK_DECISION = "WAITING_BANK_DECISION"
	BANK_REJECT_CREDIT_OFFER = "BANK_REJECT_CREDIT_OFFER"
	CUSTOMER_REJECT_CREDIT_OFFER = "CUSTOMER_REJECT_CREDIT_OFFER"
	CREDIT_OFFER_FAILED = "CREDIT_OFFER_FAILED"
	AWAIT_DELIVERY_DATES_CONFIRMATION = "AWAIT_DELIVERY_DATES_CONFIRMATION"
	SERVICE_FAULT = "SERVICE_FAULT"
	DELIVERY_SERVICE_RECEIVED = "DELIVERY_SERVICE_RECEIVED"
	USER_RECEIVED = "USER_RECEIVED"
	WAITING_FOR_STOCKS = "WAITING_FOR_STOCKS"
	AS_PART_OF_MULTI_ORDER = "AS_PART_OF_MULTI_ORDER"
	READY_FOR_LAST_MILE = "READY_FOR_LAST_MILE"
	LAST_MILE_STARTED = "LAST_MILE_STARTED"
	ANTIFRAUD = "ANTIFRAUD"
	DELIVERY_USER_NOT_RECEIVED = "DELIVERY_USER_NOT_RECEIVED"
	DELIVERY_SERVICE_DELIVERED = "DELIVERY_SERVICE_DELIVERED"
	DELIVERED_USER_NOT_RECEIVED = "DELIVERED_USER_NOT_RECEIVED"
	USER_WANTED_ANOTHER_PAYMENT_METHOD = "USER_WANTED_ANOTHER_PAYMENT_METHOD"
	USER_RECEIVED_TECHNICAL_ERROR = "USER_RECEIVED_TECHNICAL_ERROR"
	USER_FORGOT_TO_USE_BONUS = "USER_FORGOT_TO_USE_BONUS"
	DELIVERY_SERVICE_NOT_RECEIVED = "DELIVERY_SERVICE_NOT_RECEIVED"
	DELIVERY_SERVICE_LOST = "DELIVERY_SERVICE_LOST"
	SHIPPED_TO_WRONG_DELIVERY_SERVICE = "SHIPPED_TO_WRONG_DELIVERY_SERVICE"
	DELIVERED_USER_RECEIVED = "DELIVERED_USER_RECEIVED"
	WAITING_TINKOFF_DECISION = "WAITING_TINKOFF_DECISION"
	COURIER_SEARCH = "COURIER_SEARCH"
	COURIER_FOUND = "COURIER_FOUND"
	COURIER_IN_TRANSIT_TO_SENDER = "COURIER_IN_TRANSIT_TO_SENDER"
	COURIER_ARRIVED_TO_SENDER = "COURIER_ARRIVED_TO_SENDER"
	COURIER_RECEIVED = "COURIER_RECEIVED"
	COURIER_NOT_FOUND = "COURIER_NOT_FOUND"
	COURIER_NOT_DELIVER_ORDER = "COURIER_NOT_DELIVER_ORDER"
	COURIER_RETURNS_ORDER = "COURIER_RETURNS_ORDER"
	COURIER_RETURNED_ORDER = "COURIER_RETURNED_ORDER"
	WAITING_USER_DELIVERY_INPUT = "WAITING_USER_DELIVERY_INPUT"
	PICKUP_SERVICE_RECEIVED = "PICKUP_SERVICE_RECEIVED"
	PICKUP_USER_RECEIVED = "PICKUP_USER_RECEIVED"
	CANCELLED_COURIER_NOT_FOUND = "CANCELLED_COURIER_NOT_FOUND"
	COURIER_NOT_COME_FOR_ORDER = "COURIER_NOT_COME_FOR_ORDER"
	DELIVERY_NOT_MANAGED_REGION = "DELIVERY_NOT_MANAGED_REGION"
	INCOMPLETE_CONTACT_INFORMATION = "INCOMPLETE_CONTACT_INFORMATION"
	INCOMPLETE_MULTI_ORDER = "INCOMPLETE_MULTI_ORDER"
	INAPPROPRIATE_WEIGHT_SIZE = "INAPPROPRIATE_WEIGHT_SIZE"
	TECHNICAL_ERROR = "TECHNICAL_ERROR"
	SORTING_CENTER_LOST = "SORTING_CENTER_LOST"
	COURIER_SEARCH_NOT_STARTED = "COURIER_SEARCH_NOT_STARTED"
	LOST = "LOST"
	AWAIT_PAYMENT = "AWAIT_PAYMENT"
	AWAIT_LAVKA_RESERVATION = "AWAIT_LAVKA_RESERVATION"
	USER_WANTS_TO_CHANGE_ADDRESS = "USER_WANTS_TO_CHANGE_ADDRESS"
	FULL_NOT_RANSOM = "FULL_NOT_RANSOM"
	PRESCRIPTION_MISMATCH = "PRESCRIPTION_MISMATCH"
	DROPOFF_LOST = "DROPOFF_LOST"
	DROPOFF_CLOSED = "DROPOFF_CLOSED"
	DELIVERY_TO_STORE_STARTED = "DELIVERY_TO_STORE_STARTED"
	USER_WANTS_TO_CHANGE_DELIVERY_DATE = "USER_WANTS_TO_CHANGE_DELIVERY_DATE"
	WRONG_ITEM_DELIVERED = "WRONG_ITEM_DELIVERED"
	DAMAGED_BOX = "DAMAGED_BOX"
	AWAIT_DELIVERY_DATES = "AWAIT_DELIVERY_DATES"
	LAST_MILE_COURIER_SEARCH = "LAST_MILE_COURIER_SEARCH"
	PICKUP_POINT_CLOSED = "PICKUP_POINT_CLOSED"
	LEGAL_INFO_CHANGED = "LEGAL_INFO_CHANGED"
	USER_HAS_NO_TIME_TO_PICKUP_ORDER = "USER_HAS_NO_TIME_TO_PICKUP_ORDER"
	DELIVERY_CUSTOMS_ARRIVED = "DELIVERY_CUSTOMS_ARRIVED"
	DELIVERY_CUSTOMS_CLEARED = "DELIVERY_CUSTOMS_CLEARED"
	FIRST_MILE_DELIVERY_SERVICE_RECEIVED = "FIRST_MILE_DELIVERY_SERVICE_RECEIVED"
	AWAIT_AUTO_DELIVERY_DATES = "AWAIT_AUTO_DELIVERY_DATES"
	AWAIT_USER_PERSONAL_DATA = "AWAIT_USER_PERSONAL_DATA"
	NO_PERSONAL_DATA_EXPIRED = "NO_PERSONAL_DATA_EXPIRED"
	CUSTOMS_PROBLEMS = "CUSTOMS_PROBLEMS"
	AWAIT_CASHIER = "AWAIT_CASHIER"
	WAITING_POSTPAID_BUDGET_RESERVATION = "WAITING_POSTPAID_BUDGET_RESERVATION"
	AWAIT_SERVICEABLE_CONFIRMATION = "AWAIT_SERVICEABLE_CONFIRMATION"
	POSTPAID_BUDGET_RESERVATION_FAILED = "POSTPAID_BUDGET_RESERVATION_FAILED"
	AWAIT_CUSTOM_PRICE_CONFIRMATION = "AWAIT_CUSTOM_PRICE_CONFIRMATION"
	READY_FOR_PICKUP = "READY_FOR_PICKUP"
	TOO_MANY_DELIVERY_DATE_CHANGES = "TOO_MANY_DELIVERY_DATE_CHANGES"
	TOO_LONG_DELIVERY = "TOO_LONG_DELIVERY"
	DEFERRED_PAYMENT = "DEFERRED_PAYMENT"
	POSTPAID_FAILED = "POSTPAID_FAILED"
	UNKNOWN = "UNKNOWN"


@allow_unknown
class OrderTaxSystem(Enum):
	ECHN = "ECHN"		# единый сельскохозяйственный налог (ЕСХН).
	ENVD = "ENVD"		# единый налог на вмененный доход (ЕНВД).
	OSN = "OSN"		# общая система налогообложения (ОСН).
	PSN = "PSN"		# патентная система налогообложения (ПСН).
	USN = "USN"		# упрощенная система налогообложения (УСН).
	USN_MINUS_COST = "USN_MINUS_COST"		# упрощенная система налогообложения, доходы, уменьшенные на величину расходов (УСН «Доходы минус расходы»).
	NPD = "NPD"		# налог на профессиональный доход (НПД).
	UNKNOWN_VALUE = "UNKNOWN_VALUE"		# неизвестное значение


@allow_unknown
class OrderBuyerType(Enum):
	PERSON = "PERSON"
	BUSINESS = "BUSINESS"


@allow_unknown
class OrderSubsidyType(Enum):
	YANDEX_CASHBACK = "YANDEX_CASHBACK"		# скидка по подписке Яндекс Плюс.
	SUBSIDY = "SUBSIDY"		# скидка Маркета (по акциям, промокодам, купонам и т. д.)
	DELIVERY = "DELIVERY"		# скидка за доставку (DBS).


@allow_unknown
class OrderVatType(Enum):
	NO_VAT = "NO_VAT"
	VAT_0 = "VAT_0"
	VAT_10 = "VAT_10"
	VAT_10_110 = "VAT_10_110"
	VAT_20 = "VAT_20"
	VAT_20_120 = "VAT_20_120"
	VAT_18 = "VAT_18"
	VAT_18_118 = "VAT_18_118"
	VAT_12 = "VAT_12"
	VAT_05 = "VAT_05"
	VAT_07 = "VAT_07"
	UNKNOWN_VALUE = "UNKNOWN_VALUE"


@allow_unknown
class OrderItemInstanceType(Enum):
	CIS = "CIS"		# КИЗ, идентификатор единицы товара в системе «Честный ЗНАК» или «ASL BELGISI» (для продавцов Market Yandex Go). Обязателен для заполнения.
	CIS_OPTIONAL = "CIS_OPTIONAL"		# КИЗ, идентификатор единицы товара в системе «Честный ЗНАК». Необязателен для заполнения, но в ближайшее время потребуется его передача.
	UIN = "UIN"		# УИН, уникальный идентификационный номер.
	RNPT = "RNPT"		# РНПТ, регистрационный номер партии товара.
	GTD = "GTD"		# номер ГТД, грузовой таможенной декларации.


@allow_unknown
class OrderItemTag(Enum):
	ULTIMA = "ULTIMA"		# премиум-товар.
	SAFE_TAG = "SAFE_TAG"		# товар с защитной меткой.
	TURBO = "TURBO"		# товар, который быстро раскупают.


@allow_unknown
class OrderDeliveryPartnerType(Enum):
	SHOP = "SHOP"		# магазин работает со службой доставки напрямую или доставляет заказы самостоятельно.
	YANDEX_MARKET = "YANDEX_MARKET"		# магазин работает со службой доставки через Маркет.
	UNKNOWN = "UNKNOWN"		# неизвестный тип.


@allow_unknown
class OrderDeliveryType(Enum):
	DELIVERY = "DELIVERY"		# курьерская доставка.
	PICKUP = "PICKUP"		# самовывоз.
	POST = "POST"		# почта.
	DIGITAL = "DIGITAL"		# для цифровых товаров.
	UNKNOWN = "UNKNOWN"		# неизвестный тип.


@allow_unknown
class OrderDeliveryDispatchType(Enum):
	BUYER = "BUYER"		# доставка покупателю.
	MARKET_BRANDED_OUTLET = "MARKET_BRANDED_OUTLET"		# доставка в пункт выдачи заказов Маркета.
	SHOP_OUTLET = "SHOP_OUTLET"		# доставка в пункт выдачи заказов магазина.
	UNKNOWN = "UNKNOWN"		# неизвестный тип.


@allow_unknown
class OrderDeliveryEacType(Enum):
	MERCHANT_TO_COURIER = "MERCHANT_TO_COURIER"
	COURIER_TO_MERCHANT = "COURIER_TO_MERCHANT"
	CHECKING_BY_MERCHANT = "CHECKING_BY_MERCHANT"


@allow_unknown
class OrderLiftType(Enum):
	NOT_NEEDED = "NOT_NEEDED"		# не требуется.
	MANUAL = "MANUAL"		# ручной.
	ELEVATOR = "ELEVATOR"		# лифт.
	CARGO_ELEVATOR = "CARGO_ELEVATOR"		# грузовой лифт.
	FREE = "FREE"		# любой из перечисленных выше, если включена опция бесплатного подъема.
	UNKNOWN = "UNKNOWN"		# неизвестный тип.


@allow_unknown
class OrderItemStatusType(Enum):
	REJECTED = "REJECTED"		# невыкупленный.
	RETURNED = "RETURNED"		# возвращенный.


@allow_unknown
class OrderPromoType(Enum):
	DIRECT_DISCOUNT = "DIRECT_DISCOUNT"
	BLUE_SET = "BLUE_SET"
	BLUE_FLASH = "BLUE_FLASH"
	GENERIC_BUNDLE = "GENERIC_BUNDLE"
	MARKET_COUPON = "MARKET_COUPON"
	MARKET_PROMOCODE = "MARKET_PROMOCODE"
	MARKET_BLUE = "MARKET_BLUE"
	MARKET_COIN = "MARKET_COIN"
	PRICE_DROP_AS_YOU_SHOP = "PRICE_DROP_AS_YOU_SHOP"
	SECRET_SALE = "SECRET_SALE"
	CHEAPEST_AS_GIFT = "CHEAPEST_AS_GIFT"
	CASHBACK = "CASHBACK"
	SPREAD_DISCOUNT_COUNT = "SPREAD_DISCOUNT_COUNT"
	SPREAD_DISCOUNT_RECEIPT = "SPREAD_DISCOUNT_RECEIPT"
	DISCOUNT_BY_PAYMENT_TYPE = "DISCOUNT_BY_PAYMENT_TYPE"
	PERCENT_DISCOUNT = "PERCENT_DISCOUNT"
	DCO_EXTRA_DISCOUNT = "DCO_EXTRA_DISCOUNT"
	UNKNOWN = "UNKNOWN"


@allow_unknown
class OrderItemSubsidyType(Enum):
	YANDEX_CASHBACK = "YANDEX_CASHBACK"		# скидка по подписке Яндекс Плюс.
	SUBSIDY = "SUBSIDY"		# скидка Маркета (по акциям, промокодам, купонам и т. д.).



from tests.fake_async_session import FakeAsyncSession
from ya_market_api.campaign.async_api import AsyncCampaignAPI
from ya_market_api.campaign.dataclass import CampaignListRequest
from ya_market_api.base.async_config import AsyncConfig

from unittest.mock import patch, Mock

import pytest


class TestAsyncCampaignAPI:
	@pytest.mark.asyncio()
	async def test_get_campaign_list(self):
		session = FakeAsyncSession("RAW DATA")
		config = AsyncConfig(session, "", business_id=1)		# type: ignore - for testing purposes
		api = AsyncCampaignAPI(config)
		request = CampaignListRequest(page=10, page_size=100)

		with patch("ya_market_api.campaign.async_api.CampaignListResponse") as CampaignListResponseMock:
			CampaignListResponseMock.model_validate_json = Mock()
			CampaignListResponseMock.model_validate_json.return_value = "DESERIALIZED DATA"

			with patch.object(api, "validate_response") as validate_response_mock:
				assert await api.get_campaign_list() == "DESERIALIZED DATA"
				CampaignListResponseMock.model_validate_json.assert_called_once_with("RAW DATA")
				validate_response_mock.assert_called_once_with(session.response)
				assert session.last_call_method == "GET"
				assert session.last_call_url == api.router.campaign_list()
				assert session.last_call_json == None
				assert session.last_call_params == {}

				assert await api.get_campaign_list(request) == "DESERIALIZED DATA"
				assert session.last_call_method == "GET"
				assert session.last_call_url == api.router.campaign_list()
				assert session.last_call_json == None
				assert session.last_call_params == {"page": 10, "pageSize": 100}



from ya_market_api.base.async_api_mixin import AsyncAPIMixin
from ya_market_api.offer.base_api import BaseOfferAPI
from ya_market_api.offer.dataclass import OfferListByBusinessRequest, OfferListByBusinessResponse

from typing import Optional


class AsyncOfferAPI(AsyncAPIMixin, BaseOfferAPI):
	async def get_offer_list_by_business(self, request: Optional[OfferListByBusinessRequest] = None) -> OfferListByBusinessResponse:
		request = request or OfferListByBusinessRequest()
		url = self.router.offer_list_by_business(self.business_id)

		async with self.session.post(
			url=url,
			params=request.model_dump_request_params(),
			json=request.model_dump_request_payload(),
		) as response:
			self.validate_response(response)
			return OfferListByBusinessResponse.model_validate_json(await response.text())



from ya_market_api.offer.router import OfferRouter


class TestOfferRouter:
	def test_offer_list_by_business(self):
		assert OfferRouter("").offer_list_by_business("512") == "/businesses/512/offer-mappings"



from ya_market_api.feedback.dataclass.comment_update import Request, RequestComment


class TestRequest:
	def test_create(self):
		request = Request.create(512, 1024, "comment-text")
		assert isinstance(request, Request)
		assert isinstance(request.comment, RequestComment)
		assert request.feedback_id == 512
		assert request.comment.id == 1024
		assert request.comment.text == "comment-text"



from ya_market_api.base.const import RegionType

from typing import List, Optional

from pydantic.main import BaseModel
from pydantic.fields import Field


class Region(BaseModel):
	id: int
	name: str
	type: RegionType
	children: List["Region"] = Field(default_factory=list)
	parent: Optional["Region"] = Field(default=None)



from ya_market_api.base.dataclass.region import Region

from typing import List

from pydantic.main import BaseModel
from pydantic.fields import Field


class Country(BaseModel):
	code: str = Field(validation_alias="countryCode")
	region: Region


class Response(BaseModel):
	countries: List[Country]
